from .assignment_admin import AssignmentAdmin
from .dosage_guideline_admin import DosageGuidelineAdmin
from .formulation_admin import FormulationAdmin
from .formulation_type_admin import FormulationTypeAdmin
from .frequency_units_admin import FrequencyUnitsAdmin
from .medication_admin import MedicationAdmin
from .route_admin import RouteAdmin
from .units_admin import UnitsAdmin



from edc_pharmacy.utils import get_codenames

# groups
DISPENSING = "DISPENSING"
DISPENSING_VIEW = "DISPENSING_VIEW"
PHARMACY = "PHARMACY"
PHARMACY_SITE = "PHARMACY_SITE"
PHARMACY_PRESCRIBER = "PHARMACY_PRESCRIBER"
PHARMACY_VIEW = "PHARMACY_VIEW"
PHARMACY_SUPER = "PHARMACY_SUPER"

# roles
CENTRAL_PHARMACIST_ROLE = "CENTRAL_PHARMACIST_ROLE"
PHARMACIST_ROLE = "PHARMACIST_ROLE"
PHARMACY_AUDITOR_ROLE = "PHARMACY_AUDITOR_ROLE"
PHARMACY_PRESCRIBER_ROLE = "PHARMACY_PRESCRIBER_ROLE"
SITE_PHARMACIST_ROLE = "SITE_PHARMACIST_ROLE"
PHARMACY_SUPER_ROLE = "PHARMACY_SUPER_ROLE"

navbar_codenames = ["edc_pharmacy.nav_pharmacy_section"]
navbar_tuples = []
for codename in navbar_codenames:
    navbar_tuples.append((codename, f"Can access {codename.split('.')[1]}"))  # noqa: PERF401


# central pharmacist
view_only_models = [
    "edc_pharmacy.confirmation",
    "edc_pharmacy.allocation",
    "edc_pharmacy.formulation",
    "edc_pharmacy.formulationtype",
    "edc_pharmacy.frequencyunits",
    "edc_pharmacy.location",
    "edc_pharmacy.medication",
    "edc_pharmacy.stock",
    "edc_pharmacy.stockavailability",
    "edc_pharmacy.assignment",
    "edc_pharmacy.subject",
    "edc_pharmacy.visitschedule",
]
pharmacy_codenames = get_codenames([], view_only_models=view_only_models)
pharmacy_codenames.extend(navbar_codenames)
pharmacy_codenames.sort()


# site pharmacist
exclude_models = [
    "edc_pharmacy.lot",
    "edc_pharmacy.assignment",
    "edc_pharmacy.order",
    "edc_pharmacy.orderitem",
    "edc_pharmacy.receive",
    "edc_pharmacy.receiveitem",
]
view_only_models = [
    "edc_pharmacy.stocktransferproxy",
    "edc_pharmacy.stocktransferitem",
    "edc_pharmacy.confirmation",
    "edc_pharmacy.allocation",
    "edc_pharmacy.allocationproxy",
    "edc_pharmacy.container",
    "edc_pharmacy.containertype",
    "edc_pharmacy.formulation",
    "edc_pharmacy.formulationtype",
    "edc_pharmacy.frequencyunits",
    "edc_pharmacy.location",
    "edc_pharmacy.medication",
    "edc_pharmacy.product",
    "edc_pharmacy.stock",
    "edc_pharmacy.stockavailability",
    "edc_pharmacy.stockadjustment",
    "edc_pharmacy.stockproxy",
    "edc_pharmacy.subject",
    "edc_pharmacy.visitschedule",
]
pharmacy_site_codenames = get_codenames(
    [], view_only_models=view_only_models, exclude_models=exclude_models
)
pharmacy_site_codenames.extend(navbar_codenames)
pharmacy_site_codenames.sort()

# prescriber
prescriber_codenames = []
for model_name in ["dosageguideline", "formulation", "medication", "rxrefill"]:
    prescriber_codenames.extend(
        [
            c
            for c in pharmacy_codenames
            if model_name in c and c.startswith("edc_pharmacy.view")
        ]
    )
for model_name in ["rx", "rxitem"]:
    prescriber_codenames.extend([c for c in pharmacy_codenames if model_name in c])
prescriber_codenames.append("edc_pharmacy.view_subject")
prescriber_codenames.sort()



from __future__ import annotations

from importlib import import_module
from typing import TYPE_CHECKING

from django.urls.conf import include, path

if TYPE_CHECKING:
    from django.urls import URLPattern


def paths_for_urlpatterns(app_name) -> list[URLPattern]:
    paths: list[URLPattern] = []
    try:
        admin_site = import_module(f"{app_name}.admin_site")
    except ModuleNotFoundError:
        pass
    else:
        paths.append(path(f"{app_name}/admin/", getattr(admin_site, f"{app_name}_admin").urls))
    paths.append(path(f"{app_name}/", include(f"{app_name}.urls")))
    return paths



from django.contrib import admin
from django.template.loader import render_to_string
from django.urls import reverse
from edc_model.admin import HistoricalModelAdminMixin
from edc_model_admin.history import SimpleHistoryAdmin
from edc_utils import to_local
from rangefilter.filters import DateRangeFilterBuilder

from ....admin_site import edc_pharmacy_history_admin
from ....models import StorageBinItem
from ...model_admin_mixin import ModelAdminMixin


@admin.register(StorageBinItem.history.model, site=edc_pharmacy_history_admin)
class StorageBinItemHistoryAdmin(
    ModelAdminMixin, HistoricalModelAdminMixin, SimpleHistoryAdmin
):
    change_list_title = "Pharmacy: Storage bin items (History)"
    change_form_title = "Pharmacy: Storage bin item (History)"
    change_list_note_url_name = "edc_pharmacy_admin:edc_pharmacy_storagebinitem_changelist"
    history_list_display = ()
    show_object_tools = True
    show_cancel = True
    list_per_page = 20

    list_display = (
        "item_id",
        "history_type",
        "formatted_history_date",
        "site__id",
        "code",
        "storage_bin_changelist",
        "revision",
        "history_id",
    )

    list_filter = (
        "history_type",
        ("history_date", DateRangeFilterBuilder()),
        ("item_datetime", DateRangeFilterBuilder()),
    )

    search_fields = (
        "code",
        "history_id",
        "storage_bin__bin_identifier",
        "item_identifier",
    )

    @admin.display(description="ITEM#", ordering="item_identifier")
    def item_id(self, obj):
        return obj.item_identifier

    @admin.display(description="Date", ordering="item_datetime")
    def item_date(self, obj):
        if obj.item_datetime:
            return to_local(obj.item_datetime).date()
        return None

    @admin.display(description="Storage bin#")
    def storage_bin_changelist(self, obj):
        url = reverse("edc_pharmacy_admin:edc_pharmacy_storagebin_changelist")
        url = f"{url}?q={obj.storage_bin.bin_identifier}"
        context = dict(
            url=url,
            label=f"{obj.storage_bin.bin_identifier}",
            title="Go to storage bin",
        )
        return render_to_string("edc_pharmacy/stock/items_as_link.html", context=context)



# Generated by Django 2.0 on 2018-01-03 11:22

from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [("edc_identifier", "0013_auto_20171230_1316")]

    operations = [
        migrations.AlterField(
            model_name="identifiermodel", name="device_id", field=models.IntegerField()
        ),
        migrations.AlterField(
            model_name="identifiermodel",
            name="identifier_type",
            field=models.CharField(max_length=100, null=True),
        ),
        migrations.AlterField(
            model_name="identifiermodel",
            name="model",
            field=models.CharField(max_length=100, null=True),
        ),
        migrations.AlterField(
            model_name="identifiermodel",
            name="name",
            field=models.CharField(max_length=100),
        ),
        migrations.AlterField(
            model_name="identifiermodel",
            name="protocol_number",
            field=models.CharField(max_length=25),
        ),
    ]



from clinicedc_constants import HIGH_PRIORITY
from django.apps import apps as django_apps
from django.core.exceptions import ObjectDoesNotExist

from edc_action_item.action_with_notification import ActionWithNotification
from edc_visit_schedule.utils import get_offschedule_models

from ..constants import (
    ADVERSE_EVENT_ADMIN_SITE,
    ADVERSE_EVENT_APP_LABEL,
    AE_FOLLOWUP_ACTION,
    AE_INITIAL_ACTION,
    DEATH_REPORT_ACTION,
)


class DeathReportAction(ActionWithNotification):
    name = DEATH_REPORT_ACTION
    display_name = "Submit Death Report"
    notification_display_name = "Death Report"
    parent_action_names = (AE_INITIAL_ACTION, AE_FOLLOWUP_ACTION)
    show_link_to_changelist = True
    show_link_to_add = True
    priority = HIGH_PRIORITY
    singleton = True
    enable_tmg_workflow = True

    reference_model = f"{ADVERSE_EVENT_APP_LABEL}.deathreport"
    death_report_tmg_model = f"{ADVERSE_EVENT_APP_LABEL}.deathreporttmg"
    admin_site_name = ADVERSE_EVENT_ADMIN_SITE

    def get_next_actions(self):
        """Adds 1 DEATHReportTMG if not yet created and
        STUDY_TERMINATION_CONCLUSION if required.
        """
        next_actions = []
        if self.enable_tmg_workflow:
            next_actions = self.append_next_death_tmg_action(next_actions)
        return self.append_next_off_schedule_action(next_actions)

    def append_next_death_tmg_action(self, next_actions):
        if self.death_report_tmg_model:
            tmg_model_cls = django_apps.get_model(self.death_report_tmg_model)
            try:
                self.action_item_model_cls().objects.get(
                    parent_action_item=self.reference_obj.action_item,
                    related_action_item=self.reference_obj.action_item,
                    action_type__name=tmg_model_cls.action_name,
                )
            except ObjectDoesNotExist:
                next_actions = [tmg_model_cls.action_name]
        return next_actions

    def append_next_off_schedule_action(self, next_actions):
        """Appends an off schedule action to the list for each
        schedule the subject is on.

        If subject was already taken off schedule, skip. For example,
        if re-saving the DeathReport, subject may have already been
        taken off schedule.
        """
        offschedule_models = get_offschedule_models(
            subject_identifier=self.subject_identifier,
            report_datetime=self.reference_obj.report_datetime,
        )
        for off_schedule_model in offschedule_models:
            off_schedule_cls = django_apps.get_model(off_schedule_model)
            try:
                off_schedule_cls.objects.get(subject_identifier=self.subject_identifier)
            except ObjectDoesNotExist:
                next_actions.append(off_schedule_cls.action_name)
        return next_actions



from uuid import uuid4

from clinicedc_constants import OPEN
from django.db import models
from django.db.models import UniqueConstraint
from django_crypto_fields.fields import EncryptedTextField

from edc_data_manager.get_data_queries import get_data_queries
from edc_model.validators import datetime_not_future
from edc_protocol.validators import datetime_not_before_study_start
from edc_sites.managers import CurrentSiteManager
from edc_utils import age, formatted_age

from ..field_mixins import VerificationFieldsMixin
from ..managers import ConsentObjectsManager
from .consent_version_model_mixin import ConsentVersionModelMixin


class ConsentModelMixin(ConsentVersionModelMixin, VerificationFieldsMixin, models.Model):
    """Mixin for a Consent model class such as SubjectConsent.

    Declare with edc_identifier's NonUniqueSubjectIdentifierModelMixin
    """

    screening_identifier = models.CharField(verbose_name="Screening identifier", max_length=50)

    screening_datetime = models.DateTimeField(
        verbose_name="Screening datetime", null=True, editable=False
    )

    model_name = models.CharField(
        verbose_name="model",
        max_length=50,
        help_text=(
            "label_lower of this model class. Will be different if "
            "instance has been added/edited via a proxy model"
        ),
        null=True,
        editable=False,
    )

    consent_datetime = models.DateTimeField(
        verbose_name="Consent date and time",
        validators=[datetime_not_before_study_start, datetime_not_future],
    )

    report_datetime = models.DateTimeField(null=True, editable=False)

    sid = models.CharField(
        verbose_name="SID",
        max_length=15,
        null=True,
        blank=True,
        editable=False,
        help_text="Used for randomization against a prepared rando-list.",
    )

    comment = EncryptedTextField(verbose_name="Comment", max_length=250, blank=True, null=True)

    dm_comment = models.CharField(
        verbose_name="Data Management comment",
        max_length=150,
        null=True,
        editable=False,
        help_text="see also edc.data manager.",
    )

    consent_identifier = models.UUIDField(
        default=uuid4,
        editable=False,
        help_text="A unique identifier for this consent instance",
    )

    objects = ConsentObjectsManager()

    on_site = CurrentSiteManager()

    def __str__(self):
        return f"{self.get_subject_identifier()} v{self.version}"

    def natural_key(self):
        return (self.get_subject_identifier_as_pk(),)

    def save(self, *args, **kwargs):
        if not self.id:
            self.model_name = self._meta.label_lower
        self.report_datetime = self.consent_datetime
        super().save(*args, **kwargs)

    def get_dob(self):
        """Returns the date of birth"""
        return self.dob

    @property
    def age_at_consent(self):
        """Returns a relativedelta."""
        return age(self.get_dob(), self.consent_datetime)

    @property
    def formatted_age_at_consent(self):
        """Returns a string representation."""
        return formatted_age(self.get_dob(), self.consent_datetime)

    @property
    def open_data_queries(self):
        return get_data_queries(
            subject_identifier=self.subject_identifier,
            model=self._meta.label_lower,
            status=OPEN,
        )

    class Meta:
        abstract = True
        verbose_name = "Subject Consent"
        verbose_name_plural = "Subject Consents"
        constraints = [
            UniqueConstraint(
                fields=["first_name", "dob", "initials", "version"],
                name="%(app_label)s_%(class)s_first_uniq",
            ),
            UniqueConstraint(
                fields=[
                    "subject_identifier",
                    "first_name",
                    "dob",
                    "initials",
                    "version",
                ],
                name="%(app_label)s_%(class)s_subject_uniq",
            ),
            UniqueConstraint(
                fields=[
                    "version",
                    "screening_identifier",
                    "subject_identifier",
                ],
                name="%(app_label)s_%(class)s_version_uniq",
            ),
        ]



from clinicedc_constants import YES

from edc_form_validators import FormValidator

from ..constants import VIOLATION
from .mixins import IncidentFormvalidatorMixin


class ProtocolDeviationViolationFormValidator(IncidentFormvalidatorMixin, FormValidator):
    def clean(self):
        self.applicable_if(VIOLATION, field="report_type", field_applicable="safety_impact")
        self.required_if(YES, field="safety_impact", field_required="safety_impact_details")
        self.applicable_if(
            VIOLATION, field="report_type", field_applicable="study_outcomes_impact"
        )
        self.required_if(
            YES,
            field="study_outcomes_impact",
            field_required="study_outcomes_impact_details",
        )

        self.required_if(VIOLATION, field="report_type", field_required="violation_datetime")

        if "violation_type" in self.cleaned_data:
            self.required_if(VIOLATION, field="report_type", field_required="violation_type")
            self.validate_other_specify(
                field="violation_type",
                other_specify_field="violation_type_other",
            )
        if "violation" in self.cleaned_data:
            self.required_if(VIOLATION, field="report_type", field_required="violation")
            self.validate_other_specify(
                field="violation",
                other_specify_field="violation_other",
            )

        self.required_if(
            VIOLATION, field="report_type", field_required="violation_description"
        )
        self.required_if(VIOLATION, field="report_type", field_required="violation_reason")

        self.validate_close_report()



from django.apps import AppConfig as DjangoApponfig
from django.core.management.color import color_style

style = color_style()


class AppConfig(DjangoApponfig):
    name = "edc_action_item"
    verbose_name = "Action Items"
    has_exportable_data = True
    include_in_administration_section = True



from django.contrib.auth.models import User
from django.core.exceptions import ObjectDoesNotExist
from django.db.models.signals import m2m_changed, post_save
from django.dispatch import receiver

from .user_profile import UserProfile


@receiver(post_save, weak=False, sender=User, dispatch_uid="update_user_profile_on_post_save")
def update_user_profile_on_post_save(sender, instance, raw, **kwargs):
    if not raw:
        try:
            instance.userprofile  # noqa: B018
        except ObjectDoesNotExist:
            UserProfile.objects.create(user=instance)


@receiver(m2m_changed, weak=False, dispatch_uid="update_user_groups_on_role_m2m_changed")
def update_user_groups_on_role_m2m_changed(sender, action, instance, pk_set, **kwargs):
    try:
        through = instance.roles.through
    except AttributeError:
        pass
    else:
        if through == sender and action in ["post_add", "post_remove"]:
            if action == "post_add":
                instance.add_groups_for_roles(pk_set)
            elif action == "post_remove":
                instance.remove_groups_for_roles(pk_set)



from .codenames import (
    account_manager,
    administration,
    auditor,
    celery_manager,
    clinic,
    clinic_super,
    everyone,
)
from .default_groups import get_default_groups
from .default_pii_models import default_pii_models
from .default_roles import get_default_roles



from clinicedc_constants import (
    AE_WITHDRAWN,
    CONTINUING_UPDATE,
    DEAD,
    DEFINITELY_RELATED,
    GRADE3,
    GRADE4,
    GRADE5,
    LTFU,
    MILD,
    MODERATE,
    NOT_APPLICABLE,
    NOT_RECOVERED,
    NOT_RELATED,
    OTHER,
    POSSIBLY_RELATED,
    PROBABLY_RELATED,
    RECOVERED,
    RECOVERED_WITH_SEQUELAE,
    RECOVERING,
    SEVERE,
    SEVERITY_INCREASED_FROM_G3,
    UNLIKELY_RELATED,
)

AE_INTENSITY = ((MILD, "Mild"), (MODERATE, "Moderate"), (SEVERE, "Severe"))

AE_REPORT_TYPE = (
    ("initial", "Initial"),
    ("follow_up", "Follow Up"),
    ("final", "Final"),
)

AE_GRADE = (
    (GRADE3, "Grade III - Severe"),
    (GRADE4, "Grade 4 - Life-threatening"),
    (GRADE5, "Grade 5 - Death"),
)

AE_GRADE_SIMPLE = (
    (GRADE4, "Grade 4 - Life-threatening"),
    (GRADE5, "Grade 5 - Death"),
    (NOT_APPLICABLE, "Not applicable"),
)

# TODO: validate Severity increased from Grade III
AE_OUTCOME = (
    (CONTINUING_UPDATE, "Continuing/Update"),
    (SEVERITY_INCREASED_FROM_G3, "Severity increased from Grade III"),
    (RECOVERED, "Recovered/Resolved"),
    (RECOVERING, "Recovering/Resolving at end of study"),
    (NOT_RECOVERED, "Not Recovered/Resolved at end of study"),
    (LTFU, "Unknown/Lost to follow-up"),
    (RECOVERED_WITH_SEQUELAE, "Recovered with sequelae"),
    (DEAD, "Death"),
    (AE_WITHDRAWN, "Adverse event report withdrawn after TMG review"),
)

CONTACT = (
    ("tel", "Telephone conversation"),
    ("home", "Home visIt"),
    ("relative_at_clinic", "Relative visited the health facility"),
    ("patient_record", "Patient record / document"),
    (OTHER, "Other"),
)

DEATH_LOCATIONS = (
    ("home", "At home"),
    ("hospital_clinic", "Hospital/clinic"),
    (OTHER, "Elsewhere, please specify"),
)

INFORMANT = (
    ("spouse", "Spouse"),
    ("Parent", "Parent"),
    ("child", "Child"),
    ("healthcare_worker", "Healthcare Worker"),
    (OTHER, "Other"),
)

SAE_REASONS = (
    (NOT_APPLICABLE, "Not applicable"),
    (DEAD, "Death"),
    ("life_threatening", "Life-threatening"),
    ("significant_disability", "Significant disability"),
    (
        "in-patient_hospitalization",
        "In-patient hospitalization or prolongation (17 or more days from study inclusion)",
    ),
    (
        "medically_important_event",
        "Medically important event (e.g. Severe thrombophlebitis, Bacteraemia, "
        "recurrence of symptoms not requiring admission, Hospital acquired "
        "pneumonia)",
    ),
)

STUDY_DRUG_RELATIONSHIP = (
    (NOT_RELATED, "Not related"),
    (UNLIKELY_RELATED, "Unlikely related"),
    (POSSIBLY_RELATED, "Possibly related"),
    (PROBABLY_RELATED, "Probably related"),
    (DEFINITELY_RELATED, "Definitely related"),
    (NOT_APPLICABLE, "Not applicable"),
)



from django_collect_offline.offline_model import OfflineModel
from django_collect_offline.site_offline_models import site_offline_models

offline_models = [
    "edc_export.objecthistory",
    "edc_export.exportplan",
    "edc_export.exportreceipt",
    "edc_export.filehistory",
    "edc_export.uploadexportreceiptfile",
]

site_offline_models.register(offline_models, OfflineModel)



from .action_item_crf_form_mixin import ActionItemCrfFormMixin
from .action_item_form import ActionItemForm
from .action_item_form_mixin import ActionItemFormMixin
from .action_type_form import ActionTypeForm



# Generated by Django 5.2.6 on 2025-09-17 16:53

import django.core.validators
import django.utils.timezone
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        (
            "edc_pharmacy",
            "0088_alter_allocation_revision_alter_assignment_revision_and_more",
        ),
    ]

    operations = [
        migrations.AlterField(
            model_name="allocation",
            name="allocated_by",
            field=models.CharField(blank=True, default="", max_length=50),
        ),
        migrations.AlterField(
            model_name="allocation",
            name="device_created",
            field=models.CharField(
                blank=True, default="", max_length=10, verbose_name="Device created"
            ),
        ),
        migrations.AlterField(
            model_name="allocation",
            name="device_modified",
            field=models.CharField(
                blank=True, default="", max_length=10, verbose_name="Device modified"
            ),
        ),
        migrations.AlterField(
            model_name="allocation",
            name="locale_created",
            field=models.CharField(
                blank=True,
                default="",
                help_text="Auto-updated by Modeladmin",
                max_length=10,
                verbose_name="Locale created",
            ),
        ),
        migrations.AlterField(
            model_name="allocation",
            name="locale_modified",
            field=models.CharField(
                blank=True,
                default="",
                help_text="Auto-updated by Modeladmin",
                max_length=10,
                verbose_name="Locale modified",
            ),
        ),
        migrations.AlterField(
            model_name="assignment",
            name="device_created",
            field=models.CharField(
                blank=True, default="", max_length=10, verbose_name="Device created"
            ),
        ),
        migrations.AlterField(
            model_name="assignment",
            name="device_modified",
            field=models.CharField(
                blank=True, default="", max_length=10, verbose_name="Device modified"
            ),
        ),
        migrations.AlterField(
            model_name="assignment",
            name="locale_created",
            field=models.CharField(
                blank=True,
                default="",
                help_text="Auto-updated by Modeladmin",
                max_length=10,
                verbose_name="Locale created",
            ),
        ),
        migrations.AlterField(
            model_name="assignment",
            name="locale_modified",
            field=models.CharField(
                blank=True,
                default="",
                help_text="Auto-updated by Modeladmin",
                max_length=10,
                verbose_name="Locale modified",
            ),
        ),
        migrations.AlterField(
            model_name="box",
            name="box_datetime",
            field=models.DateTimeField(default=django.utils.timezone.now),
        ),
        migrations.AlterField(
            model_name="box",
            name="description",
            field=models.TextField(default=""),
        ),
        migrations.AlterField(
            model_name="box",
            name="device_created",
            field=models.CharField(
                blank=True, default="", max_length=10, verbose_name="Device created"
            ),
        ),
        migrations.AlterField(
            model_name="box",
            name="device_modified",
            field=models.CharField(
                blank=True, default="", max_length=10, verbose_name="Device modified"
            ),
        ),
        migrations.AlterField(
            model_name="box",
            name="locale_created",
            field=models.CharField(
                blank=True,
                default="",
                help_text="Auto-updated by Modeladmin",
                max_length=10,
                verbose_name="Locale created",
            ),
        ),
        migrations.AlterField(
            model_name="box",
            name="locale_modified",
            field=models.CharField(
                blank=True,
                default="",
                help_text="Auto-updated by Modeladmin",
                max_length=10,
                verbose_name="Locale modified",
            ),
        ),
        migrations.AlterField(
            model_name="confirmation",
            name="confirmed_by",
            field=models.CharField(blank=True, default="", max_length=100),
        ),
        migrations.AlterField(
            model_name="confirmation",
            name="device_created",
            field=models.CharField(
                blank=True, default="", max_length=10, verbose_name="Device created"
            ),
        ),
        migrations.AlterField(
            model_name="confirmation",
            name="device_modified",
            field=models.CharField(
                blank=True, default="", max_length=10, verbose_name="Device modified"
            ),
        ),
        migrations.AlterField(
            model_name="confirmation",
            name="locale_created",
            field=models.CharField(
                blank=True,
                default="",
                help_text="Auto-updated by Modeladmin",
                max_length=10,
                verbose_name="Locale created",
            ),
        ),
        migrations.AlterField(
            model_name="confirmation",
            name="locale_modified",
            field=models.CharField(
                blank=True,
                default="",
                help_text="Auto-updated by Modeladmin",
                max_length=10,
                verbose_name="Locale modified",
            ),
        ),
        migrations.AlterField(
            model_name="confirmationatsite",
            name="comments",
            field=models.TextField(blank=True, default=""),
        ),
        migrations.AlterField(
            model_name="confirmationatsite",
            name="device_created",
            field=models.CharField(
                blank=True, default="", max_length=10, verbose_name="Device created"
            ),
        ),
        migrations.AlterField(
            model_name="confirmationatsite",
            name="device_modified",
            field=models.CharField(
                blank=True, default="", max_length=10, verbose_name="Device modified"
            ),
        ),
        migrations.AlterField(
            model_name="confirmationatsite",
            name="locale_created",
            field=models.CharField(
                blank=True,
                default="",
                help_text="Auto-updated by Modeladmin",
                max_length=10,
                verbose_name="Locale created",
            ),
        ),
        migrations.AlterField(
            model_name="confirmationatsite",
            name="locale_modified",
            field=models.CharField(
                blank=True,
                default="",
                help_text="Auto-updated by Modeladmin",
                max_length=10,
                verbose_name="Locale modified",
            ),
        ),
        migrations.AlterField(
            model_name="confirmationatsiteitem",
            name="confirmed_by",
            field=models.CharField(blank=True, default="", max_length=150),
        ),
        migrations.AlterField(
            model_name="confirmationatsiteitem",
            name="device_created",
            field=models.CharField(
                blank=True, default="", max_length=10, verbose_name="Device created"
            ),
        ),
        migrations.AlterField(
            model_name="confirmationatsiteitem",
            name="device_modified",
            field=models.CharField(
                blank=True, default="", max_length=10, verbose_name="Device modified"
            ),
        ),
        migrations.AlterField(
            model_name="confirmationatsiteitem",
            name="locale_created",
            field=models.CharField(
                blank=True,
                default="",
                help_text="Auto-updated by Modeladmin",
                max_length=10,
                verbose_name="Locale created",
            ),
        ),
        migrations.AlterField(
            model_name="confirmationatsiteitem",
            name="locale_modified",
            field=models.CharField(
                blank=True,
                default="",
                help_text="Auto-updated by Modeladmin",
                max_length=10,
                verbose_name="Locale modified",
            ),
        ),
        migrations.AlterField(
            model_name="container",
            name="device_created",
            field=models.CharField(
                blank=True, default="", max_length=10, verbose_name="Device created"
            ),
        ),
        migrations.AlterField(
            model_name="container",
            name="device_modified",
            field=models.CharField(
                blank=True, default="", max_length=10, verbose_name="Device modified"
            ),
        ),
        migrations.AlterField(
            model_name="container",
            name="locale_created",
            field=models.CharField(
                blank=True,
                default="",
                help_text="Auto-updated by Modeladmin",
                max_length=10,
                verbose_name="Locale created",
            ),
        ),
        migrations.AlterField(
            model_name="container",
            name="locale_modified",
            field=models.CharField(
                blank=True,
                default="",
                help_text="Auto-updated by Modeladmin",
                max_length=10,
                verbose_name="Locale modified",
            ),
        ),
        migrations.AlterField(
            model_name="containertype",
            name="extra_value",
            field=models.CharField(default="", max_length=250),
        ),
        migrations.AlterField(
            model_name="containertype",
            name="field_name",
            field=models.CharField(
                blank=True,
                default="",
                editable=False,
                help_text="Not required",
                max_length=25,
            ),
        ),
        migrations.AlterField(
            model_name="containertype",
            name="plural_name",
            field=models.CharField(
                default="", max_length=250, verbose_name="Plural name"
            ),
        ),
        migrations.AlterField(
            model_name="containerunits",
            name="extra_value",
            field=models.CharField(default="", max_length=250),
        ),
        migrations.AlterField(
            model_name="containerunits",
            name="field_name",
            field=models.CharField(
                blank=True,
                default="",
                editable=False,
                help_text="Not required",
                max_length=25,
            ),
        ),
        migrations.AlterField(
            model_name="containerunits",
            name="plural_name",
            field=models.CharField(
                default="", max_length=250, verbose_name="Plural name"
            ),
        ),
        migrations.AlterField(
            model_name="dispense",
            name="crf_field_name",
            field=models.CharField(blank=True, default="", max_length=100),
        ),
        migrations.AlterField(
            model_name="dispense",
            name="crf_label_lower",
            field=models.CharField(blank=True, default="", max_length=100),
        ),
        migrations.AlterField(
            model_name="dispense",
            name="device_created",
            field=models.CharField(
                blank=True, default="", max_length=10, verbose_name="Device created"
            ),
        ),
        migrations.AlterField(
            model_name="dispense",
            name="device_modified",
            field=models.CharField(
                blank=True, default="", max_length=10, verbose_name="Device modified"
            ),
        ),
        migrations.AlterField(
            model_name="dispense",
            name="dispensed_by",
            field=models.CharField(blank=True, default="", max_length=100),
        ),
        migrations.AlterField(
            model_name="dispense",
            name="locale_created",
            field=models.CharField(
                blank=True,
                default="",
                help_text="Auto-updated by Modeladmin",
                max_length=10,
                verbose_name="Locale created",
            ),
        ),
        migrations.AlterField(
            model_name="dispense",
            name="locale_modified",
            field=models.CharField(
                blank=True,
                default="",
                help_text="Auto-updated by Modeladmin",
                max_length=10,
                verbose_name="Locale modified",
            ),
        ),
        migrations.AlterField(
            model_name="dispenseitem",
            name="device_created",
            field=models.CharField(
                blank=True, default="", max_length=10, verbose_name="Device created"
            ),
        ),
        migrations.AlterField(
            model_name="dispenseitem",
            name="device_modified",
            field=models.CharField(
                blank=True, default="", max_length=10, verbose_name="Device modified"
            ),
        ),
        migrations.AlterField(
            model_name="dispenseitem",
            name="locale_created",
            field=models.CharField(
                blank=True,
                default="",
                help_text="Auto-updated by Modeladmin",
                max_length=10,
                verbose_name="Locale created",
            ),
        ),
        migrations.AlterField(
            model_name="dispenseitem",
            name="locale_modified",
            field=models.CharField(
                blank=True,
                default="",
                help_text="Auto-updated by Modeladmin",
                max_length=10,
                verbose_name="Locale modified",
            ),
        ),
        migrations.AlterField(
            model_name="dispenseitem",
            name="visit_code",
            field=models.CharField(default="", editable=False, max_length=25),
        ),
        migrations.AlterField(
            model_name="dosageguideline",
            name="device_created",
            field=models.CharField(
                blank=True, default="", max_length=10, verbose_name="Device created"
            ),
        ),
        migrations.AlterField(
            model_name="dosageguideline",
            name="device_modified",
            field=models.CharField(
                blank=True, default="", max_length=10, verbose_name="Device modified"
            ),
        ),
        migrations.AlterField(
            model_name="dosageguideline",
            name="locale_created",
            field=models.CharField(
                blank=True,
                default="",
                help_text="Auto-updated by Modeladmin",
                max_length=10,
                verbose_name="Locale created",
            ),
        ),
        migrations.AlterField(
            model_name="dosageguideline",
            name="locale_modified",
            field=models.CharField(
                blank=True,
                default="",
                help_text="Auto-updated by Modeladmin",
                max_length=10,
                verbose_name="Locale modified",
            ),
        ),
        migrations.AlterField(
            model_name="edcpermissions",
            name="device_created",
            field=models.CharField(
                blank=True, default="", max_length=10, verbose_name="Device created"
            ),
        ),
        migrations.AlterField(
            model_name="edcpermissions",
            name="device_modified",
            field=models.CharField(
                blank=True, default="", max_length=10, verbose_name="Device modified"
            ),
        ),
        migrations.AlterField(
            model_name="edcpermissions",
            name="locale_created",
            field=models.CharField(
                blank=True,
                default="",
                help_text="Auto-updated by Modeladmin",
                max_length=10,
                verbose_name="Locale created",
            ),
        ),
        migrations.AlterField(
            model_name="edcpermissions",
            name="locale_modified",
            field=models.CharField(
                blank=True,
                default="",
                help_text="Auto-updated by Modeladmin",
                max_length=10,
                verbose_name="Locale modified",
            ),
        ),
        migrations.AlterField(
            model_name="formulation",
            name="description",
            field=models.CharField(blank=True, default="", max_length=250),
        ),
        migrations.AlterField(
            model_name="formulation",
            name="device_created",
            field=models.CharField(
                blank=True, default="", max_length=10, verbose_name="Device created"
            ),
        ),
        migrations.AlterField(
            model_name="formulation",
            name="device_modified",
            field=models.CharField(
                blank=True, default="", max_length=10, verbose_name="Device modified"
            ),
        ),
        migrations.AlterField(
            model_name="formulation",
            name="imp_description",
            field=models.CharField(blank=True, default="", max_length=250),
        ),
        migrations.AlterField(
            model_name="formulation",
            name="locale_created",
            field=models.CharField(
                blank=True,
                default="",
                help_text="Auto-updated by Modeladmin",
                max_length=10,
                verbose_name="Locale created",
            ),
        ),
        migrations.AlterField(
            model_name="formulation",
            name="locale_modified",
            field=models.CharField(
                blank=True,
                default="",
                help_text="Auto-updated by Modeladmin",
                max_length=10,
                verbose_name="Locale modified",
            ),
        ),
        migrations.AlterField(
            model_name="formulation",
            name="notes",
            field=models.TextField(blank=True, default="", max_length=250),
        ),
        migrations.AlterField(
            model_name="formulationtype",
            name="extra_value",
            field=models.CharField(default="", max_length=250),
        ),
        migrations.AlterField(
            model_name="formulationtype",
            name="field_name",
            field=models.CharField(
                blank=True,
                default="",
                editable=False,
                help_text="Not required",
                max_length=25,
            ),
        ),
        migrations.AlterField(
            model_name="formulationtype",
            name="plural_name",
            field=models.CharField(
                default="", max_length=250, verbose_name="Plural name"
            ),
        ),
        migrations.AlterField(
            model_name="frequencyunits",
            name="extra_value",
            field=models.CharField(default="", max_length=250),
        ),
        migrations.AlterField(
            model_name="frequencyunits",
            name="field_name",
            field=models.CharField(
                blank=True,
                default="",
                editable=False,
                help_text="Not required",
                max_length=25,
            ),
        ),
        migrations.AlterField(
            model_name="frequencyunits",
            name="plural_name",
            field=models.CharField(
                default="", max_length=250, verbose_name="Plural name"
            ),
        ),
        migrations.AlterField(
            model_name="historicalallocation",
            name="allocated_by",
            field=models.CharField(blank=True, default="", max_length=50),
        ),
        migrations.AlterField(
            model_name="historicalallocation",
            name="device_created",
            field=models.CharField(
                blank=True, default="", max_length=10, verbose_name="Device created"
            ),
        ),
        migrations.AlterField(
            model_name="historicalallocation",
            name="device_modified",
            field=models.CharField(
                blank=True, default="", max_length=10, verbose_name="Device modified"
            ),
        ),
        migrations.AlterField(
            model_name="historicalallocation",
            name="locale_created",
            field=models.CharField(
                blank=True,
                default="",
                help_text="Auto-updated by Modeladmin",
                max_length=10,
                verbose_name="Locale created",
            ),
        ),
        migrations.AlterField(
            model_name="historicalallocation",
            name="locale_modified",
            field=models.CharField(
                blank=True,
                default="",
                help_text="Auto-updated by Modeladmin",
                max_length=10,
                verbose_name="Locale modified",
            ),
        ),
        migrations.AlterField(
            model_name="historicalassignment",
            name="device_created",
            field=models.CharField(
                blank=True, default="", max_length=10, verbose_name="Device created"
            ),
        ),
        migrations.AlterField(
            model_name="historicalassignment",
            name="device_modified",
            field=models.CharField(
                blank=True, default="", max_length=10, verbose_name="Device modified"
            ),
        ),
        migrations.AlterField(
            model_name="historicalassignment",
            name="locale_created",
            field=models.CharField(
                blank=True,
                default="",
                help_text="Auto-updated by Modeladmin",
                max_length=10,
                verbose_name="Locale created",
            ),
        ),
        migrations.AlterField(
            model_name="historicalassignment",
            name="locale_modified",
            field=models.CharField(
                blank=True,
                default="",
                help_text="Auto-updated by Modeladmin",
                max_length=10,
                verbose_name="Locale modified",
            ),
        ),
        migrations.AlterField(
            model_name="historicalconfirmation",
            name="confirmed_by",
            field=models.CharField(blank=True, default="", max_length=100),
        ),
        migrations.AlterField(
            model_name="historicalconfirmation",
            name="device_created",
            field=models.CharField(
                blank=True, default="", max_length=10, verbose_name="Device created"
            ),
        ),
        migrations.AlterField(
            model_name="historicalconfirmation",
            name="device_modified",
            field=models.CharField(
                blank=True, default="", max_length=10, verbose_name="Device modified"
            ),
        ),
        migrations.AlterField(
            model_name="historicalconfirmation",
            name="locale_created",
            field=models.CharField(
                blank=True,
                default="",
                help_text="Auto-updated by Modeladmin",
                max_length=10,
                verbose_name="Locale created",
            ),
        ),
        migrations.AlterField(
            model_name="historicalconfirmation",
            name="locale_modified",
            field=models.CharField(
                blank=True,
                default="",
                help_text="Auto-updated by Modeladmin",
                max_length=10,
                verbose_name="Locale modified",
            ),
        ),
        migrations.AlterField(
            model_name="historicalconfirmationatsite",
            name="comments",
            field=models.TextField(blank=True, default=""),
        ),
        migrations.AlterField(
            model_name="historicalconfirmationatsite",
            name="device_created",
            field=models.CharField(
                blank=True, default="", max_length=10, verbose_name="Device created"
            ),
        ),
        migrations.AlterField(
            model_name="historicalconfirmationatsite",
            name="device_modified",
            field=models.CharField(
                blank=True, default="", max_length=10, verbose_name="Device modified"
            ),
        ),
        migrations.AlterField(
            model_name="historicalconfirmationatsite",
            name="locale_created",
            field=models.CharField(
                blank=True,
                default="",
                help_text="Auto-updated by Modeladmin",
                max_length=10,
                verbose_name="Locale created",
            ),
        ),
        migrations.AlterField(
            model_name="historicalconfirmationatsite",
            name="locale_modified",
            field=models.CharField(
                blank=True,
                default="",
                help_text="Auto-updated by Modeladmin",
                max_length=10,
                verbose_name="Locale modified",
            ),
        ),
        migrations.AlterField(
            model_name="historicalconfirmationatsiteitem",
            name="confirmed_by",
            field=models.CharField(blank=True, default="", max_length=150),
        ),
        migrations.AlterField(
            model_name="historicalconfirmationatsiteitem",
            name="device_created",
            field=models.CharField(
                blank=True, default="", max_length=10, verbose_name="Device created"
            ),
        ),
        migrations.AlterField(
            model_name="historicalconfirmationatsiteitem",
            name="device_modified",
            field=models.CharField(
                blank=True, default="", max_length=10, verbose_name="Device modified"
            ),
        ),
        migrations.AlterField(
            model_name="historicalconfirmationatsiteitem",
            name="locale_created",
            field=models.CharField(
                blank=True,
                default="",
                help_text="Auto-updated by Modeladmin",
                max_length=10,
                verbose_name="Locale created",
            ),
        ),
        migrations.AlterField(
            model_name="historicalconfirmationatsiteitem",
            name="locale_modified",
            field=models.CharField(
                blank=True,
                default="",
                help_text="Auto-updated by Modeladmin",
                max_length=10,
                verbose_name="Locale modified",
            ),
        ),
        migrations.AlterField(
            model_name="historicalcontainer",
            name="device_created",
            field=models.CharField(
                blank=True, default="", max_length=10, verbose_name="Device created"
            ),
        ),
        migrations.AlterField(
            model_name="historicalcontainer",
            name="device_modified",
            field=models.CharField(
                blank=True, default="", max_length=10, verbose_name="Device modified"
            ),
        ),
        migrations.AlterField(
            model_name="historicalcontainer",
            name="locale_created",
            field=models.CharField(
                blank=True,
                default="",
                help_text="Auto-updated by Modeladmin",
                max_length=10,
                verbose_name="Locale created",
            ),
        ),
        migrations.AlterField(
            model_name="historicalcontainer",
            name="locale_modified",
            field=models.CharField(
                blank=True,
                default="",
                help_text="Auto-updated by Modeladmin",
                max_length=10,
                verbose_name="Locale modified",
            ),
        ),
        migrations.AlterField(
            model_name="historicaldispense",
            name="crf_field_name",
            field=models.CharField(blank=True, default="", max_length=100),
        ),
        migrations.AlterField(
            model_name="historicaldispense",
            name="crf_label_lower",
            field=models.CharField(blank=True, default="", max_length=100),
        ),
        migrations.AlterField(
            model_name="historicaldispense",
            name="device_created",
            field=models.CharField(
                blank=True, default="", max_length=10, verbose_name="Device created"
            ),
        ),
        migrations.AlterField(
            model_name="historicaldispense",
            name="device_modified",
            field=models.CharField(
                blank=True, default="", max_length=10, verbose_name="Device modified"
            ),
        ),
        migrations.AlterField(
            model_name="historicaldispense",
            name="dispensed_by",
            field=models.CharField(blank=True, default="", max_length=100),
        ),
        migrations.AlterField(
            model_name="historicaldispense",
            name="locale_created",
            field=models.CharField(
                blank=True,
                default="",
                help_text="Auto-updated by Modeladmin",
                max_length=10,
                verbose_name="Locale created",
            ),
        ),
        migrations.AlterField(
            model_name="historicaldispense",
            name="locale_modified",
            field=models.CharField(
                blank=True,
                default="",
                help_text="Auto-updated by Modeladmin",
                max_length=10,
                verbose_name="Locale modified",
            ),
        ),
        migrations.AlterField(
            model_name="historicaldispenseitem",
            name="device_created",
            field=models.CharField(
                blank=True, default="", max_length=10, verbose_name="Device created"
            ),
        ),
        migrations.AlterField(
            model_name="historicaldispenseitem",
            name="device_modified",
            field=models.CharField(
                blank=True, default="", max_length=10, verbose_name="Device modified"
            ),
        ),
        migrations.AlterField(
            model_name="historicaldispenseitem",
            name="locale_created",
            field=models.CharField(
                blank=True,
                default="",
                help_text="Auto-updated by Modeladmin",
                max_length=10,
                verbose_name="Locale created",
            ),
        ),
        migrations.AlterField(
            model_name="historicaldispenseitem",
            name="locale_modified",
            field=models.CharField(
                blank=True,
                default="",
                help_text="Auto-updated by Modeladmin",
                max_length=10,
                verbose_name="Locale modified",
            ),
        ),
        migrations.AlterField(
            model_name="historicaldispenseitem",
            name="visit_code",
            field=models.CharField(default="", editable=False, max_length=25),
        ),
        migrations.AlterField(
            model_name="historicaldosageguideline",
            name="device_created",
            field=models.CharField(
                blank=True, default="", max_length=10, verbose_name="Device created"
            ),
        ),
        migrations.AlterField(
            model_name="historicaldosageguideline",
            name="device_modified",
            field=models.CharField(
                blank=True, default="", max_length=10, verbose_name="Device modified"
            ),
        ),
        migrations.AlterField(
            model_name="historicaldosageguideline",
            name="locale_created",
            field=models.CharField(
                blank=True,
                default="",
                help_text="Auto-updated by Modeladmin",
                max_length=10,
                verbose_name="Locale created",
            ),
        ),
        migrations.AlterField(
            model_name="historicaldosageguideline",
            name="locale_modified",
            field=models.CharField(
                blank=True,
                default="",
                help_text="Auto-updated by Modeladmin",
                max_length=10,
                verbose_name="Locale modified",
            ),
        ),
        migrations.AlterField(
            model_name="historicalformulation",
            name="description",
            field=models.CharField(blank=True, default="", max_length=250),
        ),
        migrations.AlterField(
            model_name="historicalformulation",
            name="device_created",
            field=models.CharField(
                blank=True, default="", max_length=10, verbose_name="Device created"
            ),
        ),
        migrations.AlterField(
            model_name="historicalformulation",
            name="device_modified",
            field=models.CharField(
                blank=True, default="", max_length=10, verbose_name="Device modified"
            ),
        ),
        migrations.AlterField(
            model_name="historicalformulation",
            name="imp_description",
            field=models.CharField(blank=True, default="", max_length=250),
        ),
        migrations.AlterField(
            model_name="historicalformulation",
            name="locale_created",
            field=models.CharField(
                blank=True,
                default="",
                help_text="Auto-updated by Modeladmin",
                max_length=10,
                verbose_name="Locale created",
            ),
        ),
        migrations.AlterField(
            model_name="historicalformulation",
            name="locale_modified",
            field=models.CharField(
                blank=True,
                default="",
                help_text="Auto-updated by Modeladmin",
                max_length=10,
                verbose_name="Locale modified",
            ),
        ),
        migrations.AlterField(
            model_name="historicalformulation",
            name="notes",
            field=models.TextField(blank=True, default="", max_length=250),
        ),
        migrations.AlterField(
            model_name="historicallocation",
            name="contact_email",
            field=models.EmailField(blank=True, default="", max_length=150),
        ),
        migrations.AlterField(
            model_name="historicallocation",
            name="contact_name",
            field=models.CharField(blank=True, default="", max_length=150),
        ),
        migrations.AlterField(
            model_name="historicallocation",
            name="contact_tel",
            field=models.CharField(
                blank=True,
                default="",
                max_length=150,
                validators=[django.core.validators.RegexValidator("[0-9]{1,15}")],
            ),
        ),
        migrations.AlterField(
            model_name="historicallocation",
            name="device_created",
            field=models.CharField(
                blank=True, default="", max_length=10, verbose_name="Device created"
            ),
        ),
        migrations.AlterField(
            model_name="historicallocation",
            name="device_modified",
            field=models.CharField(
                blank=True, default="", max_length=10, verbose_name="Device modified"
            ),
        ),
        migrations.AlterField(
            model_name="historicallocation",
            name="extra_value",
            field=models.CharField(default="", max_length=250),
        ),
        migrations.AlterField(
            model_name="historicallocation",
            name="field_name",
            field=models.CharField(
                blank=True,
                default="",
                editable=False,
                help_text="Not required",
                max_length=25,
            ),
        ),
        migrations.AlterField(
            model_name="historicallocation",
            name="locale_created",
            field=models.CharField(
                blank=True,
                default="",
                help_text="Auto-updated by Modeladmin",
                max_length=10,
                verbose_name="Locale created",
            ),
        ),
        migrations.AlterField(
            model_name="historicallocation",
            name="locale_modified",
            field=models.CharField(
                blank=True,
                default="",
                help_text="Auto-updated by Modeladmin",
                max_length=10,
                verbose_name="Locale modified",
            ),
        ),
        migrations.AlterField(
            model_name="historicallocation",
            name="plural_name",
            field=models.CharField(
                default="", max_length=250, verbose_name="Plural name"
            ),
        ),
        migrations.AlterField(
            model_name="historicallot",
            name="comment",
            field=models.CharField(blank=True, default="", max_length=250),
        ),
        migrations.AlterField(
            model_name="historicallot",
            name="country_of_origin",
            field=models.CharField(blank=True, default="", max_length=50),
        ),
        migrations.AlterField(
            model_name="historicallot",
            name="device_created",
            field=models.CharField(
                blank=True, default="", max_length=10, verbose_name="Device created"
            ),
        ),
        migrations.AlterField(
            model_name="historicallot",
            name="device_modified",
            field=models.CharField(
                blank=True, default="", max_length=10, verbose_name="Device modified"
            ),
        ),
        migrations.AlterField(
            model_name="historicallot",
            name="locale_created",
            field=models.CharField(
                blank=True,
                default="",
                help_text="Auto-updated by Modeladmin",
                max_length=10,
                verbose_name="Locale created",
            ),
        ),
        migrations.AlterField(
            model_name="historicallot",
            name="locale_modified",
            field=models.CharField(
                blank=True,
                default="",
                help_text="Auto-updated by Modeladmin",
                max_length=10,
                verbose_name="Locale modified",
            ),
        ),
        migrations.AlterField(
            model_name="historicallot",
            name="manufactured_by",
            field=models.CharField(blank=True, default="", max_length=50),
        ),
        migrations.AlterField(
            model_name="historicallot",
            name="reference",
            field=models.CharField(blank=True, default="", max_length=50),
        ),
        migrations.AlterField(
            model_name="historicallot",
            name="storage_conditions",
            field=models.CharField(blank=True, default="", max_length=50),
        ),
        migrations.AlterField(
            model_name="historicalmedication",
            name="device_created",
            field=models.CharField(
                blank=True, default="", max_length=10, verbose_name="Device created"
            ),
        ),
        migrations.AlterField(
            model_name="historicalmedication",
            name="device_modified",
            field=models.CharField(
                blank=True, default="", max_length=10, verbose_name="Device modified"
            ),
        ),
        migrations.AlterField(
            model_name="historicalmedication",
            name="locale_created",
            field=models.CharField(
                blank=True,
                default="",
                help_text="Auto-updated by Modeladmin",
                max_length=10,
                verbose_name="Locale created",
            ),
        ),
        migrations.AlterField(
            model_name="historicalmedication",
            name="locale_modified",
            field=models.CharField(
                blank=True,
                default="",
                help_text="Auto-updated by Modeladmin",
                max_length=10,
                verbose_name="Locale modified",
            ),
        ),
        migrations.AlterField(
            model_name="historicalmedication",
            name="notes",
            field=models.TextField(blank=True, default="", max_length=250),
        ),
        migrations.AlterField(
            model_name="historicalorder",
            name="comment",
            field=models.TextField(blank=True, default=""),
        ),
        migrations.AlterField(
            model_name="historicalorder",
            name="device_created",
            field=models.CharField(
                blank=True, default="", max_length=10, verbose_name="Device created"
            ),
        ),
        migrations.AlterField(
            model_name="historicalorder",
            name="device_modified",
            field=models.CharField(
                blank=True, default="", max_length=10, verbose_name="Device modified"
            ),
        ),
        migrations.AlterField(
            model_name="historicalorder",
            name="locale_created",
            field=models.CharField(
                blank=True,
                default="",
                help_text="Auto-updated by Modeladmin",
                max_length=10,
                verbose_name="Locale created",
            ),
        ),
        migrations.AlterField(
            model_name="historicalorder",
            name="locale_modified",
            field=models.CharField(
                blank=True,
                default="",
                help_text="Auto-updated by Modeladmin",
                max_length=10,
                verbose_name="Locale modified",
            ),
        ),
        migrations.AlterField(
            model_name="historicalorder",
            name="title",
            field=models.CharField(
                default="", help_text="A short description of this order", max_length=50
            ),
        ),
        migrations.AlterField(
            model_name="historicalorderitem",
            name="device_created",
            field=models.CharField(
                blank=True, default="", max_length=10, verbose_name="Device created"
            ),
        ),
        migrations.AlterField(
            model_name="historicalorderitem",
            name="device_modified",
            field=models.CharField(
                blank=True, default="", max_length=10, verbose_name="Device modified"
            ),
        ),
        migrations.AlterField(
            model_name="historicalorderitem",
            name="locale_created",
            field=models.CharField(
                blank=True,
                default="",
                help_text="Auto-updated by Modeladmin",
                max_length=10,
                verbose_name="Locale created",
            ),
        ),
        migrations.AlterField(
            model_name="historicalorderitem",
            name="locale_modified",
            field=models.CharField(
                blank=True,
                default="",
                help_text="Auto-updated by Modeladmin",
                max_length=10,
                verbose_name="Locale modified",
            ),
        ),
        migrations.AlterField(
            model_name="historicalproduct",
            name="device_created",
            field=models.CharField(
                blank=True, default="", max_length=10, verbose_name="Device created"
            ),
        ),
        migrations.AlterField(
            model_name="historicalproduct",
            name="device_modified",
            field=models.CharField(
                blank=True, default="", max_length=10, verbose_name="Device modified"
            ),
        ),
        migrations.AlterField(
            model_name="historicalproduct",
            name="locale_created",
            field=models.CharField(
                blank=True,
                default="",
                help_text="Auto-updated by Modeladmin",
                max_length=10,
                verbose_name="Locale created",
            ),
        ),
        migrations.AlterField(
            model_name="historicalproduct",
            name="locale_modified",
            field=models.CharField(
                blank=True,
                default="",
                help_text="Auto-updated by Modeladmin",
                max_length=10,
                verbose_name="Locale modified",
            ),
        ),
        migrations.AlterField(
            model_name="historicalreceive",
            name="comment",
            field=models.TextField(blank=True, default=""),
        ),
        migrations.AlterField(
            model_name="historicalreceive",
            name="device_created",
            field=models.CharField(
                blank=True, default="", max_length=10, verbose_name="Device created"
            ),
        ),
        migrations.AlterField(
            model_name="historicalreceive",
            name="device_modified",
            field=models.CharField(
                blank=True, default="", max_length=10, verbose_name="Device modified"
            ),
        ),
        migrations.AlterField(
            model_name="historicalreceive",
            name="invoice_number",
            field=models.CharField(blank=True, default="", max_length=50),
        ),
        migrations.AlterField(
            model_name="historicalreceive",
            name="locale_created",
            field=models.CharField(
                blank=True,
                default="",
                help_text="Auto-updated by Modeladmin",
                max_length=10,
                verbose_name="Locale created",
            ),
        ),
        migrations.AlterField(
            model_name="historicalreceive",
            name="locale_modified",
            field=models.CharField(
                blank=True,
                default="",
                help_text="Auto-updated by Modeladmin",
                max_length=10,
                verbose_name="Locale modified",
            ),
        ),
        migrations.AlterField(
            model_name="historicalreceiveitem",
            name="comment",
            field=models.TextField(blank=True, default=""),
        ),
        migrations.AlterField(
            model_name="historicalreceiveitem",
            name="device_created",
            field=models.CharField(
                blank=True, default="", max_length=10, verbose_name="Device created"
            ),
        ),
        migrations.AlterField(
            model_name="historicalreceiveitem",
            name="device_modified",
            field=models.CharField(
                blank=True, default="", max_length=10, verbose_name="Device modified"
            ),
        ),
        migrations.AlterField(
            model_name="historicalreceiveitem",
            name="locale_created",
            field=models.CharField(
                blank=True,
                default="",
                help_text="Auto-updated by Modeladmin",
                max_length=10,
                verbose_name="Locale created",
            ),
        ),
        migrations.AlterField(
            model_name="historicalreceiveitem",
            name="locale_modified",
            field=models.CharField(
                blank=True,
                default="",
                help_text="Auto-updated by Modeladmin",
                max_length=10,
                verbose_name="Locale modified",
            ),
        ),
        migrations.AlterField(
            model_name="historicalreceiveitem",
            name="name",
            field=models.CharField(
                blank=True,
                default="",
                help_text="Leave blank to use default",
                max_length=200,
            ),
        ),
        migrations.AlterField(
            model_name="historicalreceiveitem",
            name="reference",
            field=models.CharField(blank=True, default="", max_length=150),
        ),
        migrations.AlterField(
            model_name="historicalrepackrequest",
            name="device_created",
            field=models.CharField(
                blank=True, default="", max_length=10, verbose_name="Device created"
            ),
        ),
        migrations.AlterField(
            model_name="historicalrepackrequest",
            name="device_modified",
            field=models.CharField(
                blank=True, default="", max_length=10, verbose_name="Device modified"
            ),
        ),
        migrations.AlterField(
            model_name="historicalrepackrequest",
            name="locale_created",
            field=models.CharField(
                blank=True,
                default="",
                help_text="Auto-updated by Modeladmin",
                max_length=10,
                verbose_name="Locale created",
            ),
        ),
        migrations.AlterField(
            model_name="historicalrepackrequest",
            name="locale_modified",
            field=models.CharField(
                blank=True,
                default="",
                help_text="Auto-updated by Modeladmin",
                max_length=10,
                verbose_name="Locale modified",
            ),
        ),
        migrations.AlterField(
            model_name="historicalrx",
            name="action_identifier",
            field=models.CharField(
                blank=True, db_index=True, default="", max_length=50
            ),
        ),
        migrations.AlterField(
            model_name="historicalrx",
            name="action_item_reason",
            field=models.TextField(default="", editable=False),
        ),
        migrations.AlterField(
            model_name="historicalrx",
            name="clinician_initials",
            field=models.CharField(default="", max_length=3),
        ),
        migrations.AlterField(
            model_name="historicalrx",
            name="device_created",
            field=models.CharField(
                blank=True, default="", max_length=10, verbose_name="Device created"
            ),
        ),
        migrations.AlterField(
            model_name="historicalrx",
            name="device_modified",
            field=models.CharField(
                blank=True, default="", max_length=10, verbose_name="Device modified"
            ),
        ),
        migrations.AlterField(
            model_name="historicalrx",
            name="locale_created",
            field=models.CharField(
                blank=True,
                default="",
                help_text="Auto-updated by Modeladmin",
                max_length=10,
                verbose_name="Locale created",
            ),
        ),
        migrations.AlterField(
            model_name="historicalrx",
            name="locale_modified",
            field=models.CharField(
                blank=True,
                default="",
                help_text="Auto-updated by Modeladmin",
                max_length=10,
                verbose_name="Locale modified",
            ),
        ),
        migrations.AlterField(
            model_name="historicalrx",
            name="notes",
            field=models.TextField(
                blank=True,
                default="",
                help_text="Private notes for pharmacist only",
                max_length=250,
            ),
        ),
        migrations.AlterField(
            model_name="historicalrx",
            name="parent_action_identifier",
            field=models.CharField(
                blank=True,
                default="",
                help_text="action identifier that links to parent reference model instance.",
                max_length=30,
            ),
        ),
        migrations.AlterField(
            model_name="historicalrx",
            name="rando_sid",
            field=models.CharField(blank=True, default="", max_length=25),
        ),
        migrations.AlterField(
            model_name="historicalrx",
            name="randomizer_name",
            field=models.CharField(blank=True, default="", max_length=25),
        ),
        migrations.AlterField(
            model_name="historicalrx",
            name="related_action_identifier",
            field=models.CharField(
                blank=True,
                default="",
                help_text="action identifier that links to related reference model instance.",
                max_length=30,
            ),
        ),
        migrations.AlterField(
            model_name="historicalrxrefill",
            name="device_created",
            field=models.CharField(
                blank=True, default="", max_length=10, verbose_name="Device created"
            ),
        ),
        migrations.AlterField(
            model_name="historicalrxrefill",
            name="device_modified",
            field=models.CharField(
                blank=True, default="", max_length=10, verbose_name="Device modified"
            ),
        ),
        migrations.AlterField(
            model_name="historicalrxrefill",
            name="locale_created",
            field=models.CharField(
                blank=True,
                default="",
                help_text="Auto-updated by Modeladmin",
                max_length=10,
                verbose_name="Locale created",
            ),
        ),
        migrations.AlterField(
            model_name="historicalrxrefill",
            name="locale_modified",
            field=models.CharField(
                blank=True,
                default="",
                help_text="Auto-updated by Modeladmin",
                max_length=10,
                verbose_name="Locale modified",
            ),
        ),
        migrations.AlterField(
            model_name="historicalrxrefill",
            name="notes",
            field=models.TextField(
                blank=True,
                default="",
                help_text="Additional information for patient",
                max_length=250,
            ),
        ),
        migrations.AlterField(
            model_name="historicalstock",
            name="confirmed_by",
            field=models.CharField(blank=True, default="", max_length=150),
        ),
        migrations.AlterField(
            model_name="historicalstock",
            name="description",
            field=models.CharField(blank=True, default="", max_length=100),
        ),
        migrations.AlterField(
            model_name="historicalstock",
            name="device_created",
            field=models.CharField(
                blank=True, default="", max_length=10, verbose_name="Device created"
            ),
        ),
        migrations.AlterField(
            model_name="historicalstock",
            name="device_modified",
            field=models.CharField(
                blank=True, default="", max_length=10, verbose_name="Device modified"
            ),
        ),
        migrations.AlterField(
            model_name="historicalstock",
            name="locale_created",
            field=models.CharField(
                blank=True,
                default="",
                help_text="Auto-updated by Modeladmin",
                max_length=10,
                verbose_name="Locale created",
            ),
        ),
        migrations.AlterField(
            model_name="historicalstock",
            name="locale_modified",
            field=models.CharField(
                blank=True,
                default="",
                help_text="Auto-updated by Modeladmin",
                max_length=10,
                verbose_name="Locale modified",
            ),
        ),
        migrations.AlterField(
            model_name="historicalstock",
            name="subject_identifier",
            field=models.CharField(blank=True, default="", max_length=50),
        ),
        migrations.AlterField(
            model_name="historicalstockadjustment",
            name="device_created",
            field=models.CharField(
                blank=True, default="", max_length=10, verbose_name="Device created"
            ),
        ),
        migrations.AlterField(
            model_name="historicalstockadjustment",
            name="device_modified",
            field=models.CharField(
                blank=True, default="", max_length=10, verbose_name="Device modified"
            ),
        ),
        migrations.AlterField(
            model_name="historicalstockadjustment",
            name="locale_created",
            field=models.CharField(
                blank=True,
                default="",
                help_text="Auto-updated by Modeladmin",
                max_length=10,
                verbose_name="Locale created",
            ),
        ),
        migrations.AlterField(
            model_name="historicalstockadjustment",
            name="locale_modified",
            field=models.CharField(
                blank=True,
                default="",
                help_text="Auto-updated by Modeladmin",
                max_length=10,
                verbose_name="Locale modified",
            ),
        ),
        migrations.AlterField(
            model_name="historicalstockadjustment",
            name="reason",
            field=models.TextField(default=""),
        ),
        migrations.AlterField(
            model_name="historicalstockrequest",
            name="cancel",
            field=models.CharField(
                blank=True,
                default="",
                max_length=6,
                verbose_name="To cancel this request, type the word 'CANCEL' here and save the form:",
            ),
        ),
        migrations.AlterField(
            model_name="historicalstockrequest",
            name="device_created",
            field=models.CharField(
                blank=True, default="", max_length=10, verbose_name="Device created"
            ),
        ),
        migrations.AlterField(
            model_name="historicalstockrequest",
            name="device_modified",
            field=models.CharField(
                blank=True, default="", max_length=10, verbose_name="Device modified"
            ),
        ),
        migrations.AlterField(
            model_name="historicalstockrequest",
            name="excluded_subject_identifiers",
            field=models.TextField(
                blank=True,
                default="",
                verbose_name="Exclude these subjects from this request. (Usually left blank)",
            ),
        ),
        migrations.AlterField(
            model_name="historicalstockrequest",
            name="labels",
            field=models.TextField(
                blank=True,
                default="",
                help_text="A cell to capture and confirm printed/scanned labels related to this Stock request. See StockRequestItem.",
                verbose_name="Labels",
            ),
        ),
        migrations.AlterField(
            model_name="historicalstockrequest",
            name="locale_created",
            field=models.CharField(
                blank=True,
                default="",
                help_text="Auto-updated by Modeladmin",
                max_length=10,
                verbose_name="Locale created",
            ),
        ),
        migrations.AlterField(
            model_name="historicalstockrequest",
            name="locale_modified",
            field=models.CharField(
                blank=True,
                default="",
                help_text="Auto-updated by Modeladmin",
                max_length=10,
                verbose_name="Locale modified",
            ),
        ),
        migrations.AlterField(
            model_name="historicalstockrequest",
            name="subject_identifiers",
            field=models.TextField(
                blank=True,
                default="",
                help_text="By adding subject identifiers in this box, only these subjects will be included in the request. All others will be ignored.",
                verbose_name="Include ONLY these subjects in this request. (Usually left blank)",
            ),
        ),
        migrations.AlterField(
            model_name="historicalstockrequestitem",
            name="device_created",
            field=models.CharField(
                blank=True, default="", max_length=10, verbose_name="Device created"
            ),
        ),
        migrations.AlterField(
            model_name="historicalstockrequestitem",
            name="device_modified",
            field=models.CharField(
                blank=True, default="", max_length=10, verbose_name="Device modified"
            ),
        ),
        migrations.AlterField(
            model_name="historicalstockrequestitem",
            name="locale_created",
            field=models.CharField(
                blank=True,
                default="",
                help_text="Auto-updated by Modeladmin",
                max_length=10,
                verbose_name="Locale created",
            ),
        ),
        migrations.AlterField(
            model_name="historicalstockrequestitem",
            name="locale_modified",
            field=models.CharField(
                blank=True,
                default="",
                help_text="Auto-updated by Modeladmin",
                max_length=10,
                verbose_name="Locale modified",
            ),
        ),
        migrations.AlterField(
            model_name="historicalstockrequestitem",
            name="visit_code",
            field=models.CharField(default="", editable=False, max_length=25),
        ),
        migrations.AlterField(
            model_name="historicalstocktransfer",
            name="device_created",
            field=models.CharField(
                blank=True, default="", max_length=10, verbose_name="Device created"
            ),
        ),
        migrations.AlterField(
            model_name="historicalstocktransfer",
            name="device_modified",
            field=models.CharField(
                blank=True, default="", max_length=10, verbose_name="Device modified"
            ),
        ),
        migrations.AlterField(
            model_name="historicalstocktransfer",
            name="locale_created",
            field=models.CharField(
                blank=True,
                default="",
                help_text="Auto-updated by Modeladmin",
                max_length=10,
                verbose_name="Locale created",
            ),
        ),
        migrations.AlterField(
            model_name="historicalstocktransfer",
            name="locale_modified",
            field=models.CharField(
                blank=True,
                default="",
                help_text="Auto-updated by Modeladmin",
                max_length=10,
                verbose_name="Locale modified",
            ),
        ),
        migrations.AlterField(
            model_name="historicalstocktransferitem",
            name="device_created",
            field=models.CharField(
                blank=True, default="", max_length=10, verbose_name="Device created"
            ),
        ),
        migrations.AlterField(
            model_name="historicalstocktransferitem",
            name="device_modified",
            field=models.CharField(
                blank=True, default="", max_length=10, verbose_name="Device modified"
            ),
        ),
        migrations.AlterField(
            model_name="historicalstocktransferitem",
            name="locale_created",
            field=models.CharField(
                blank=True,
                default="",
                help_text="Auto-updated by Modeladmin",
                max_length=10,
                verbose_name="Locale created",
            ),
        ),
        migrations.AlterField(
            model_name="historicalstocktransferitem",
            name="locale_modified",
            field=models.CharField(
                blank=True,
                default="",
                help_text="Auto-updated by Modeladmin",
                max_length=10,
                verbose_name="Locale modified",
            ),
        ),
        migrations.AlterField(
            model_name="historicalstoragebin",
            name="description",
            field=models.CharField(blank=True, default="", max_length=100),
        ),
        migrations.AlterField(
            model_name="historicalstoragebin",
            name="device_created",
            field=models.CharField(
                blank=True, default="", max_length=10, verbose_name="Device created"
            ),
        ),
        migrations.AlterField(
            model_name="historicalstoragebin",
            name="device_modified",
            field=models.CharField(
                blank=True, default="", max_length=10, verbose_name="Device modified"
            ),
        ),
        migrations.AlterField(
            model_name="historicalstoragebin",
            name="locale_created",
            field=models.CharField(
                blank=True,
                default="",
                help_text="Auto-updated by Modeladmin",
                max_length=10,
                verbose_name="Locale created",
            ),
        ),
        migrations.AlterField(
            model_name="historicalstoragebin",
            name="locale_modified",
            field=models.CharField(
                blank=True,
                default="",
                help_text="Auto-updated by Modeladmin",
                max_length=10,
                verbose_name="Locale modified",
            ),
        ),
        migrations.AlterField(
            model_name="historicalstoragebinitem",
            name="device_created",
            field=models.CharField(
                blank=True, default="", max_length=10, verbose_name="Device created"
            ),
        ),
        migrations.AlterField(
            model_name="historicalstoragebinitem",
            name="device_modified",
            field=models.CharField(
                blank=True, default="", max_length=10, verbose_name="Device modified"
            ),
        ),
        migrations.AlterField(
            model_name="historicalstoragebinitem",
            name="locale_created",
            field=models.CharField(
                blank=True,
                default="",
                help_text="Auto-updated by Modeladmin",
                max_length=10,
                verbose_name="Locale created",
            ),
        ),
        migrations.AlterField(
            model_name="historicalstoragebinitem",
            name="locale_modified",
            field=models.CharField(
                blank=True,
                default="",
                help_text="Auto-updated by Modeladmin",
                max_length=10,
                verbose_name="Locale modified",
            ),
        ),
        migrations.AlterField(
            model_name="historicalsupplier",
            name="address_one",
            field=models.CharField(blank=True, default="", max_length=255),
        ),
        migrations.AlterField(
            model_name="historicalsupplier",
            name="address_two",
            field=models.CharField(blank=True, default="", max_length=255),
        ),
        migrations.AlterField(
            model_name="historicalsupplier",
            name="city",
            field=models.CharField(blank=True, default="", max_length=255),
        ),
        migrations.AlterField(
            model_name="historicalsupplier",
            name="contact",
            field=models.CharField(blank=True, default="", max_length=255),
        ),
        migrations.AlterField(
            model_name="historicalsupplier",
            name="country",
            field=models.CharField(blank=True, default="", max_length=255),
        ),
        migrations.AlterField(
            model_name="historicalsupplier",
            name="device_created",
            field=models.CharField(
                blank=True, default="", max_length=10, verbose_name="Device created"
            ),
        ),
        migrations.AlterField(
            model_name="historicalsupplier",
            name="device_modified",
            field=models.CharField(
                blank=True, default="", max_length=10, verbose_name="Device modified"
            ),
        ),
        migrations.AlterField(
            model_name="historicalsupplier",
            name="email",
            field=models.EmailField(blank=True, default="", max_length=254),
        ),
        migrations.AlterField(
            model_name="historicalsupplier",
            name="email_alternative",
            field=models.EmailField(blank=True, default="", max_length=254),
        ),
        migrations.AlterField(
            model_name="historicalsupplier",
            name="locale_created",
            field=models.CharField(
                blank=True,
                default="",
                help_text="Auto-updated by Modeladmin",
                max_length=10,
                verbose_name="Locale created",
            ),
        ),
        migrations.AlterField(
            model_name="historicalsupplier",
            name="locale_modified",
            field=models.CharField(
                blank=True,
                default="",
                help_text="Auto-updated by Modeladmin",
                max_length=10,
                verbose_name="Locale modified",
            ),
        ),
        migrations.AlterField(
            model_name="historicalsupplier",
            name="postal_code",
            field=models.CharField(blank=True, default="", max_length=255),
        ),
        migrations.AlterField(
            model_name="historicalsupplier",
            name="state",
            field=models.CharField(blank=True, default="", max_length=255),
        ),
        migrations.AlterField(
            model_name="historicalsupplier",
            name="telephone",
            field=models.CharField(blank=True, default="", max_length=15),
        ),
        migrations.AlterField(
            model_name="historicalsupplier",
            name="telephone_alternative",
            field=models.CharField(blank=True, default="", max_length=15),
        ),
        migrations.AlterField(
            model_name="location",
            name="contact_email",
            field=models.EmailField(blank=True, default="", max_length=150),
        ),
        migrations.AlterField(
            model_name="location",
            name="contact_name",
            field=models.CharField(blank=True, default="", max_length=150),
        ),
        migrations.AlterField(
            model_name="location",
            name="contact_tel",
            field=models.CharField(
                blank=True,
                default="",
                max_length=150,
                validators=[django.core.validators.RegexValidator("[0-9]{1,15}")],
            ),
        ),
        migrations.AlterField(
            model_name="location",
            name="device_created",
            field=models.CharField(
                blank=True, default="", max_length=10, verbose_name="Device created"
            ),
        ),
        migrations.AlterField(
            model_name="location",
            name="device_modified",
            field=models.CharField(
                blank=True, default="", max_length=10, verbose_name="Device modified"
            ),
        ),
        migrations.AlterField(
            model_name="location",
            name="extra_value",
            field=models.CharField(default="", max_length=250),
        ),
        migrations.AlterField(
            model_name="location",
            name="field_name",
            field=models.CharField(
                blank=True,
                default="",
                editable=False,
                help_text="Not required",
                max_length=25,
            ),
        ),
        migrations.AlterField(
            model_name="location",
            name="locale_created",
            field=models.CharField(
                blank=True,
                default="",
                help_text="Auto-updated by Modeladmin",
                max_length=10,
                verbose_name="Locale created",
            ),
        ),
        migrations.AlterField(
            model_name="location",
            name="locale_modified",
            field=models.CharField(
                blank=True,
                default="",
                help_text="Auto-updated by Modeladmin",
                max_length=10,
                verbose_name="Locale modified",
            ),
        ),
        migrations.AlterField(
            model_name="location",
            name="plural_name",
            field=models.CharField(
                default="", max_length=250, verbose_name="Plural name"
            ),
        ),
        migrations.AlterField(
            model_name="lot",
            name="comment",
            field=models.CharField(blank=True, default="", max_length=250),
        ),
        migrations.AlterField(
            model_name="lot",
            name="country_of_origin",
            field=models.CharField(blank=True, default="", max_length=50),
        ),
        migrations.AlterField(
            model_name="lot",
            name="device_created",
            field=models.CharField(
                blank=True, default="", max_length=10, verbose_name="Device created"
            ),
        ),
        migrations.AlterField(
            model_name="lot",
            name="device_modified",
            field=models.CharField(
                blank=True, default="", max_length=10, verbose_name="Device modified"
            ),
        ),
        migrations.AlterField(
            model_name="lot",
            name="locale_created",
            field=models.CharField(
                blank=True,
                default="",
                help_text="Auto-updated by Modeladmin",
                max_length=10,
                verbose_name="Locale created",
            ),
        ),
        migrations.AlterField(
            model_name="lot",
            name="locale_modified",
            field=models.CharField(
                blank=True,
                default="",
                help_text="Auto-updated by Modeladmin",
                max_length=10,
                verbose_name="Locale modified",
            ),
        ),
        migrations.AlterField(
            model_name="lot",
            name="manufactured_by",
            field=models.CharField(blank=True, default="", max_length=50),
        ),
        migrations.AlterField(
            model_name="lot",
            name="reference",
            field=models.CharField(blank=True, default="", max_length=50),
        ),
        migrations.AlterField(
            model_name="lot",
            name="storage_conditions",
            field=models.CharField(blank=True, default="", max_length=50),
        ),
        migrations.AlterField(
            model_name="medication",
            name="device_created",
            field=models.CharField(
                blank=True, default="", max_length=10, verbose_name="Device created"
            ),
        ),
        migrations.AlterField(
            model_name="medication",
            name="device_modified",
            field=models.CharField(
                blank=True, default="", max_length=10, verbose_name="Device modified"
            ),
        ),
        migrations.AlterField(
            model_name="medication",
            name="locale_created",
            field=models.CharField(
                blank=True,
                default="",
                help_text="Auto-updated by Modeladmin",
                max_length=10,
                verbose_name="Locale created",
            ),
        ),
        migrations.AlterField(
            model_name="medication",
            name="locale_modified",
            field=models.CharField(
                blank=True,
                default="",
                help_text="Auto-updated by Modeladmin",
                max_length=10,
                verbose_name="Locale modified",
            ),
        ),
        migrations.AlterField(
            model_name="medication",
            name="notes",
            field=models.TextField(blank=True, default="", max_length=250),
        ),
        migrations.AlterField(
            model_name="order",
            name="comment",
            field=models.TextField(blank=True, default=""),
        ),
        migrations.AlterField(
            model_name="order",
            name="device_created",
            field=models.CharField(
                blank=True, default="", max_length=10, verbose_name="Device created"
            ),
        ),
        migrations.AlterField(
            model_name="order",
            name="device_modified",
            field=models.CharField(
                blank=True, default="", max_length=10, verbose_name="Device modified"
            ),
        ),
        migrations.AlterField(
            model_name="order",
            name="locale_created",
            field=models.CharField(
                blank=True,
                default="",
                help_text="Auto-updated by Modeladmin",
                max_length=10,
                verbose_name="Locale created",
            ),
        ),
        migrations.AlterField(
            model_name="order",
            name="locale_modified",
            field=models.CharField(
                blank=True,
                default="",
                help_text="Auto-updated by Modeladmin",
                max_length=10,
                verbose_name="Locale modified",
            ),
        ),
        migrations.AlterField(
            model_name="order",
            name="title",
            field=models.CharField(
                default="", help_text="A short description of this order", max_length=50
            ),
        ),
        migrations.AlterField(
            model_name="orderitem",
            name="device_created",
            field=models.CharField(
                blank=True, default="", max_length=10, verbose_name="Device created"
            ),
        ),
        migrations.AlterField(
            model_name="orderitem",
            name="device_modified",
            field=models.CharField(
                blank=True, default="", max_length=10, verbose_name="Device modified"
            ),
        ),
        migrations.AlterField(
            model_name="orderitem",
            name="locale_created",
            field=models.CharField(
                blank=True,
                default="",
                help_text="Auto-updated by Modeladmin",
                max_length=10,
                verbose_name="Locale created",
            ),
        ),
        migrations.AlterField(
            model_name="orderitem",
            name="locale_modified",
            field=models.CharField(
                blank=True,
                default="",
                help_text="Auto-updated by Modeladmin",
                max_length=10,
                verbose_name="Locale modified",
            ),
        ),
        migrations.AlterField(
            model_name="product",
            name="device_created",
            field=models.CharField(
                blank=True, default="", max_length=10, verbose_name="Device created"
            ),
        ),
        migrations.AlterField(
            model_name="product",
            name="device_modified",
            field=models.CharField(
                blank=True, default="", max_length=10, verbose_name="Device modified"
            ),
        ),
        migrations.AlterField(
            model_name="product",
            name="locale_created",
            field=models.CharField(
                blank=True,
                default="",
                help_text="Auto-updated by Modeladmin",
                max_length=10,
                verbose_name="Locale created",
            ),
        ),
        migrations.AlterField(
            model_name="product",
            name="locale_modified",
            field=models.CharField(
                blank=True,
                default="",
                help_text="Auto-updated by Modeladmin",
                max_length=10,
                verbose_name="Locale modified",
            ),
        ),
        migrations.AlterField(
            model_name="receive",
            name="comment",
            field=models.TextField(blank=True, default=""),
        ),
        migrations.AlterField(
            model_name="receive",
            name="device_created",
            field=models.CharField(
                blank=True, default="", max_length=10, verbose_name="Device created"
            ),
        ),
        migrations.AlterField(
            model_name="receive",
            name="device_modified",
            field=models.CharField(
                blank=True, default="", max_length=10, verbose_name="Device modified"
            ),
        ),
        migrations.AlterField(
            model_name="receive",
            name="invoice_number",
            field=models.CharField(blank=True, default="", max_length=50),
        ),
        migrations.AlterField(
            model_name="receive",
            name="locale_created",
            field=models.CharField(
                blank=True,
                default="",
                help_text="Auto-updated by Modeladmin",
                max_length=10,
                verbose_name="Locale created",
            ),
        ),
        migrations.AlterField(
            model_name="receive",
            name="locale_modified",
            field=models.CharField(
                blank=True,
                default="",
                help_text="Auto-updated by Modeladmin",
                max_length=10,
                verbose_name="Locale modified",
            ),
        ),
        migrations.AlterField(
            model_name="receiveitem",
            name="comment",
            field=models.TextField(blank=True, default=""),
        ),
        migrations.AlterField(
            model_name="receiveitem",
            name="device_created",
            field=models.CharField(
                blank=True, default="", max_length=10, verbose_name="Device created"
            ),
        ),
        migrations.AlterField(
            model_name="receiveitem",
            name="device_modified",
            field=models.CharField(
                blank=True, default="", max_length=10, verbose_name="Device modified"
            ),
        ),
        migrations.AlterField(
            model_name="receiveitem",
            name="locale_created",
            field=models.CharField(
                blank=True,
                default="",
                help_text="Auto-updated by Modeladmin",
                max_length=10,
                verbose_name="Locale created",
            ),
        ),
        migrations.AlterField(
            model_name="receiveitem",
            name="locale_modified",
            field=models.CharField(
                blank=True,
                default="",
                help_text="Auto-updated by Modeladmin",
                max_length=10,
                verbose_name="Locale modified",
            ),
        ),
        migrations.AlterField(
            model_name="receiveitem",
            name="name",
            field=models.CharField(
                blank=True,
                default="",
                help_text="Leave blank to use default",
                max_length=200,
            ),
        ),
        migrations.AlterField(
            model_name="receiveitem",
            name="reference",
            field=models.CharField(blank=True, default="", max_length=150),
        ),
        migrations.AlterField(
            model_name="repackrequest",
            name="device_created",
            field=models.CharField(
                blank=True, default="", max_length=10, verbose_name="Device created"
            ),
        ),
        migrations.AlterField(
            model_name="repackrequest",
            name="device_modified",
            field=models.CharField(
                blank=True, default="", max_length=10, verbose_name="Device modified"
            ),
        ),
        migrations.AlterField(
            model_name="repackrequest",
            name="locale_created",
            field=models.CharField(
                blank=True,
                default="",
                help_text="Auto-updated by Modeladmin",
                max_length=10,
                verbose_name="Locale created",
            ),
        ),
        migrations.AlterField(
            model_name="repackrequest",
            name="locale_modified",
            field=models.CharField(
                blank=True,
                default="",
                help_text="Auto-updated by Modeladmin",
                max_length=10,
                verbose_name="Locale modified",
            ),
        ),
        migrations.AlterField(
            model_name="room",
            name="description",
            field=models.TextField(default=""),
        ),
        migrations.AlterField(
            model_name="room",
            name="device_created",
            field=models.CharField(
                blank=True, default="", max_length=10, verbose_name="Device created"
            ),
        ),
        migrations.AlterField(
            model_name="room",
            name="device_modified",
            field=models.CharField(
                blank=True, default="", max_length=10, verbose_name="Device modified"
            ),
        ),
        migrations.AlterField(
            model_name="room",
            name="locale_created",
            field=models.CharField(
                blank=True,
                default="",
                help_text="Auto-updated by Modeladmin",
                max_length=10,
                verbose_name="Locale created",
            ),
        ),
        migrations.AlterField(
            model_name="room",
            name="locale_modified",
            field=models.CharField(
                blank=True,
                default="",
                help_text="Auto-updated by Modeladmin",
                max_length=10,
                verbose_name="Locale modified",
            ),
        ),
        migrations.AlterField(
            model_name="room",
            name="room_datetime",
            field=models.DateTimeField(default=django.utils.timezone.now),
        ),
        migrations.AlterField(
            model_name="route",
            name="extra_value",
            field=models.CharField(default="", max_length=250),
        ),
        migrations.AlterField(
            model_name="route",
            name="field_name",
            field=models.CharField(
                blank=True,
                default="",
                editable=False,
                help_text="Not required",
                max_length=25,
            ),
        ),
        migrations.AlterField(
            model_name="route",
            name="plural_name",
            field=models.CharField(
                default="", max_length=250, verbose_name="Plural name"
            ),
        ),
        migrations.AlterField(
            model_name="rx",
            name="action_identifier",
            field=models.CharField(blank=True, default="", max_length=50, unique=True),
        ),
        migrations.AlterField(
            model_name="rx",
            name="action_item_reason",
            field=models.TextField(default="", editable=False),
        ),
        migrations.AlterField(
            model_name="rx",
            name="clinician_initials",
            field=models.CharField(default="", max_length=3),
        ),
        migrations.AlterField(
            model_name="rx",
            name="device_created",
            field=models.CharField(
                blank=True, default="", max_length=10, verbose_name="Device created"
            ),
        ),
        migrations.AlterField(
            model_name="rx",
            name="device_modified",
            field=models.CharField(
                blank=True, default="", max_length=10, verbose_name="Device modified"
            ),
        ),
        migrations.AlterField(
            model_name="rx",
            name="locale_created",
            field=models.CharField(
                blank=True,
                default="",
                help_text="Auto-updated by Modeladmin",
                max_length=10,
                verbose_name="Locale created",
            ),
        ),
        migrations.AlterField(
            model_name="rx",
            name="locale_modified",
            field=models.CharField(
                blank=True,
                default="",
                help_text="Auto-updated by Modeladmin",
                max_length=10,
                verbose_name="Locale modified",
            ),
        ),
        migrations.AlterField(
            model_name="rx",
            name="notes",
            field=models.TextField(
                blank=True,
                default="",
                help_text="Private notes for pharmacist only",
                max_length=250,
            ),
        ),
        migrations.AlterField(
            model_name="rx",
            name="parent_action_identifier",
            field=models.CharField(
                blank=True,
                default="",
                help_text="action identifier that links to parent reference model instance.",
                max_length=30,
            ),
        ),
        migrations.AlterField(
            model_name="rx",
            name="rando_sid",
            field=models.CharField(blank=True, default="", max_length=25),
        ),
        migrations.AlterField(
            model_name="rx",
            name="randomizer_name",
            field=models.CharField(blank=True, default="", max_length=25),
        ),
        migrations.AlterField(
            model_name="rx",
            name="related_action_identifier",
            field=models.CharField(
                blank=True,
                default="",
                help_text="action identifier that links to related reference model instance.",
                max_length=30,
            ),
        ),
        migrations.AlterField(
            model_name="rxrefill",
            name="device_created",
            field=models.CharField(
                blank=True, default="", max_length=10, verbose_name="Device created"
            ),
        ),
        migrations.AlterField(
            model_name="rxrefill",
            name="device_modified",
            field=models.CharField(
                blank=True, default="", max_length=10, verbose_name="Device modified"
            ),
        ),
        migrations.AlterField(
            model_name="rxrefill",
            name="locale_created",
            field=models.CharField(
                blank=True,
                default="",
                help_text="Auto-updated by Modeladmin",
                max_length=10,
                verbose_name="Locale created",
            ),
        ),
        migrations.AlterField(
            model_name="rxrefill",
            name="locale_modified",
            field=models.CharField(
                blank=True,
                default="",
                help_text="Auto-updated by Modeladmin",
                max_length=10,
                verbose_name="Locale modified",
            ),
        ),
        migrations.AlterField(
            model_name="rxrefill",
            name="notes",
            field=models.TextField(
                blank=True,
                default="",
                help_text="Additional information for patient",
                max_length=250,
            ),
        ),
        migrations.AlterField(
            model_name="scanduplicates",
            name="device_created",
            field=models.CharField(
                blank=True, default="", max_length=10, verbose_name="Device created"
            ),
        ),
        migrations.AlterField(
            model_name="scanduplicates",
            name="device_modified",
            field=models.CharField(
                blank=True, default="", max_length=10, verbose_name="Device modified"
            ),
        ),
        migrations.AlterField(
            model_name="scanduplicates",
            name="locale_created",
            field=models.CharField(
                blank=True,
                default="",
                help_text="Auto-updated by Modeladmin",
                max_length=10,
                verbose_name="Locale created",
            ),
        ),
        migrations.AlterField(
            model_name="scanduplicates",
            name="locale_modified",
            field=models.CharField(
                blank=True,
                default="",
                help_text="Auto-updated by Modeladmin",
                max_length=10,
                verbose_name="Locale modified",
            ),
        ),
        migrations.AlterField(
            model_name="shelf",
            name="description",
            field=models.TextField(default=""),
        ),
        migrations.AlterField(
            model_name="shelf",
            name="device_created",
            field=models.CharField(
                blank=True, default="", max_length=10, verbose_name="Device created"
            ),
        ),
        migrations.AlterField(
            model_name="shelf",
            name="device_modified",
            field=models.CharField(
                blank=True, default="", max_length=10, verbose_name="Device modified"
            ),
        ),
        migrations.AlterField(
            model_name="shelf",
            name="locale_created",
            field=models.CharField(
                blank=True,
                default="",
                help_text="Auto-updated by Modeladmin",
                max_length=10,
                verbose_name="Locale created",
            ),
        ),
        migrations.AlterField(
            model_name="shelf",
            name="locale_modified",
            field=models.CharField(
                blank=True,
                default="",
                help_text="Auto-updated by Modeladmin",
                max_length=10,
                verbose_name="Locale modified",
            ),
        ),
        migrations.AlterField(
            model_name="shelf",
            name="shelf_datetime",
            field=models.DateTimeField(default=django.utils.timezone.now),
        ),
        migrations.AlterField(
            model_name="stock",
            name="confirmed_by",
            field=models.CharField(blank=True, default="", max_length=150),
        ),
        migrations.AlterField(
            model_name="stock",
            name="description",
            field=models.CharField(blank=True, default="", max_length=100),
        ),
        migrations.AlterField(
            model_name="stock",
            name="device_created",
            field=models.CharField(
                blank=True, default="", max_length=10, verbose_name="Device created"
            ),
        ),
        migrations.AlterField(
            model_name="stock",
            name="device_modified",
            field=models.CharField(
                blank=True, default="", max_length=10, verbose_name="Device modified"
            ),
        ),
        migrations.AlterField(
            model_name="stock",
            name="locale_created",
            field=models.CharField(
                blank=True,
                default="",
                help_text="Auto-updated by Modeladmin",
                max_length=10,
                verbose_name="Locale created",
            ),
        ),
        migrations.AlterField(
            model_name="stock",
            name="locale_modified",
            field=models.CharField(
                blank=True,
                default="",
                help_text="Auto-updated by Modeladmin",
                max_length=10,
                verbose_name="Locale modified",
            ),
        ),
        migrations.AlterField(
            model_name="stock",
            name="subject_identifier",
            field=models.CharField(blank=True, default="", max_length=50),
        ),
        migrations.AlterField(
            model_name="stockadjustment",
            name="device_created",
            field=models.CharField(
                blank=True, default="", max_length=10, verbose_name="Device created"
            ),
        ),
        migrations.AlterField(
            model_name="stockadjustment",
            name="device_modified",
            field=models.CharField(
                blank=True, default="", max_length=10, verbose_name="Device modified"
            ),
        ),
        migrations.AlterField(
            model_name="stockadjustment",
            name="locale_created",
            field=models.CharField(
                blank=True,
                default="",
                help_text="Auto-updated by Modeladmin",
                max_length=10,
                verbose_name="Locale created",
            ),
        ),
        migrations.AlterField(
            model_name="stockadjustment",
            name="locale_modified",
            field=models.CharField(
                blank=True,
                default="",
                help_text="Auto-updated by Modeladmin",
                max_length=10,
                verbose_name="Locale modified",
            ),
        ),
        migrations.AlterField(
            model_name="stockadjustment",
            name="reason",
            field=models.TextField(default=""),
        ),
        migrations.AlterField(
            model_name="stockavailability",
            name="bins",
            field=models.TextField(default=""),
        ),
        migrations.AlterField(
            model_name="stockavailability",
            name="codes",
            field=models.TextField(default=""),
        ),
        migrations.AlterField(
            model_name="stockavailability",
            name="subject_identifier",
            field=models.CharField(default="", max_length=50),
        ),
        migrations.AlterField(
            model_name="stockrequest",
            name="cancel",
            field=models.CharField(
                blank=True,
                default="",
                max_length=6,
                verbose_name="To cancel this request, type the word 'CANCEL' here and save the form:",
            ),
        ),
        migrations.AlterField(
            model_name="stockrequest",
            name="device_created",
            field=models.CharField(
                blank=True, default="", max_length=10, verbose_name="Device created"
            ),
        ),
        migrations.AlterField(
            model_name="stockrequest",
            name="device_modified",
            field=models.CharField(
                blank=True, default="", max_length=10, verbose_name="Device modified"
            ),
        ),
        migrations.AlterField(
            model_name="stockrequest",
            name="excluded_subject_identifiers",
            field=models.TextField(
                blank=True,
                default="",
                verbose_name="Exclude these subjects from this request. (Usually left blank)",
            ),
        ),
        migrations.AlterField(
            model_name="stockrequest",
            name="labels",
            field=models.TextField(
                blank=True,
                default="",
                help_text="A cell to capture and confirm printed/scanned labels related to this Stock request. See StockRequestItem.",
                verbose_name="Labels",
            ),
        ),
        migrations.AlterField(
            model_name="stockrequest",
            name="locale_created",
            field=models.CharField(
                blank=True,
                default="",
                help_text="Auto-updated by Modeladmin",
                max_length=10,
                verbose_name="Locale created",
            ),
        ),
        migrations.AlterField(
            model_name="stockrequest",
            name="locale_modified",
            field=models.CharField(
                blank=True,
                default="",
                help_text="Auto-updated by Modeladmin",
                max_length=10,
                verbose_name="Locale modified",
            ),
        ),
        migrations.AlterField(
            model_name="stockrequest",
            name="subject_identifiers",
            field=models.TextField(
                blank=True,
                default="",
                help_text="By adding subject identifiers in this box, only these subjects will be included in the request. All others will be ignored.",
                verbose_name="Include ONLY these subjects in this request. (Usually left blank)",
            ),
        ),
        migrations.AlterField(
            model_name="stockrequestitem",
            name="device_created",
            field=models.CharField(
                blank=True, default="", max_length=10, verbose_name="Device created"
            ),
        ),
        migrations.AlterField(
            model_name="stockrequestitem",
            name="device_modified",
            field=models.CharField(
                blank=True, default="", max_length=10, verbose_name="Device modified"
            ),
        ),
        migrations.AlterField(
            model_name="stockrequestitem",
            name="locale_created",
            field=models.CharField(
                blank=True,
                default="",
                help_text="Auto-updated by Modeladmin",
                max_length=10,
                verbose_name="Locale created",
            ),
        ),
        migrations.AlterField(
            model_name="stockrequestitem",
            name="locale_modified",
            field=models.CharField(
                blank=True,
                default="",
                help_text="Auto-updated by Modeladmin",
                max_length=10,
                verbose_name="Locale modified",
            ),
        ),
        migrations.AlterField(
            model_name="stockrequestitem",
            name="visit_code",
            field=models.CharField(default="", editable=False, max_length=25),
        ),
        migrations.AlterField(
            model_name="stocktransfer",
            name="device_created",
            field=models.CharField(
                blank=True, default="", max_length=10, verbose_name="Device created"
            ),
        ),
        migrations.AlterField(
            model_name="stocktransfer",
            name="device_modified",
            field=models.CharField(
                blank=True, default="", max_length=10, verbose_name="Device modified"
            ),
        ),
        migrations.AlterField(
            model_name="stocktransfer",
            name="locale_created",
            field=models.CharField(
                blank=True,
                default="",
                help_text="Auto-updated by Modeladmin",
                max_length=10,
                verbose_name="Locale created",
            ),
        ),
        migrations.AlterField(
            model_name="stocktransfer",
            name="locale_modified",
            field=models.CharField(
                blank=True,
                default="",
                help_text="Auto-updated by Modeladmin",
                max_length=10,
                verbose_name="Locale modified",
            ),
        ),
        migrations.AlterField(
            model_name="stocktransferitem",
            name="device_created",
            field=models.CharField(
                blank=True, default="", max_length=10, verbose_name="Device created"
            ),
        ),
        migrations.AlterField(
            model_name="stocktransferitem",
            name="device_modified",
            field=models.CharField(
                blank=True, default="", max_length=10, verbose_name="Device modified"
            ),
        ),
        migrations.AlterField(
            model_name="stocktransferitem",
            name="locale_created",
            field=models.CharField(
                blank=True,
                default="",
                help_text="Auto-updated by Modeladmin",
                max_length=10,
                verbose_name="Locale created",
            ),
        ),
        migrations.AlterField(
            model_name="stocktransferitem",
            name="locale_modified",
            field=models.CharField(
                blank=True,
                default="",
                help_text="Auto-updated by Modeladmin",
                max_length=10,
                verbose_name="Locale modified",
            ),
        ),
        migrations.AlterField(
            model_name="storagebin",
            name="description",
            field=models.CharField(blank=True, default="", max_length=100),
        ),
        migrations.AlterField(
            model_name="storagebin",
            name="device_created",
            field=models.CharField(
                blank=True, default="", max_length=10, verbose_name="Device created"
            ),
        ),
        migrations.AlterField(
            model_name="storagebin",
            name="device_modified",
            field=models.CharField(
                blank=True, default="", max_length=10, verbose_name="Device modified"
            ),
        ),
        migrations.AlterField(
            model_name="storagebin",
            name="locale_created",
            field=models.CharField(
                blank=True,
                default="",
                help_text="Auto-updated by Modeladmin",
                max_length=10,
                verbose_name="Locale created",
            ),
        ),
        migrations.AlterField(
            model_name="storagebin",
            name="locale_modified",
            field=models.CharField(
                blank=True,
                default="",
                help_text="Auto-updated by Modeladmin",
                max_length=10,
                verbose_name="Locale modified",
            ),
        ),
        migrations.AlterField(
            model_name="storagebinitem",
            name="device_created",
            field=models.CharField(
                blank=True, default="", max_length=10, verbose_name="Device created"
            ),
        ),
        migrations.AlterField(
            model_name="storagebinitem",
            name="device_modified",
            field=models.CharField(
                blank=True, default="", max_length=10, verbose_name="Device modified"
            ),
        ),
        migrations.AlterField(
            model_name="storagebinitem",
            name="locale_created",
            field=models.CharField(
                blank=True,
                default="",
                help_text="Auto-updated by Modeladmin",
                max_length=10,
                verbose_name="Locale created",
            ),
        ),
        migrations.AlterField(
            model_name="storagebinitem",
            name="locale_modified",
            field=models.CharField(
                blank=True,
                default="",
                help_text="Auto-updated by Modeladmin",
                max_length=10,
                verbose_name="Locale modified",
            ),
        ),
        migrations.AlterField(
            model_name="supplier",
            name="address_one",
            field=models.CharField(blank=True, default="", max_length=255),
        ),
        migrations.AlterField(
            model_name="supplier",
            name="address_two",
            field=models.CharField(blank=True, default="", max_length=255),
        ),
        migrations.AlterField(
            model_name="supplier",
            name="city",
            field=models.CharField(blank=True, default="", max_length=255),
        ),
        migrations.AlterField(
            model_name="supplier",
            name="contact",
            field=models.CharField(blank=True, default="", max_length=255),
        ),
        migrations.AlterField(
            model_name="supplier",
            name="country",
            field=models.CharField(blank=True, default="", max_length=255),
        ),
        migrations.AlterField(
            model_name="supplier",
            name="device_created",
            field=models.CharField(
                blank=True, default="", max_length=10, verbose_name="Device created"
            ),
        ),
        migrations.AlterField(
            model_name="supplier",
            name="device_modified",
            field=models.CharField(
                blank=True, default="", max_length=10, verbose_name="Device modified"
            ),
        ),
        migrations.AlterField(
            model_name="supplier",
            name="email",
            field=models.EmailField(blank=True, default="", max_length=254),
        ),
        migrations.AlterField(
            model_name="supplier",
            name="email_alternative",
            field=models.EmailField(blank=True, default="", max_length=254),
        ),
        migrations.AlterField(
            model_name="supplier",
            name="locale_created",
            field=models.CharField(
                blank=True,
                default="",
                help_text="Auto-updated by Modeladmin",
                max_length=10,
                verbose_name="Locale created",
            ),
        ),
        migrations.AlterField(
            model_name="supplier",
            name="locale_modified",
            field=models.CharField(
                blank=True,
                default="",
                help_text="Auto-updated by Modeladmin",
                max_length=10,
                verbose_name="Locale modified",
            ),
        ),
        migrations.AlterField(
            model_name="supplier",
            name="postal_code",
            field=models.CharField(blank=True, default="", max_length=255),
        ),
        migrations.AlterField(
            model_name="supplier",
            name="state",
            field=models.CharField(blank=True, default="", max_length=255),
        ),
        migrations.AlterField(
            model_name="supplier",
            name="telephone",
            field=models.CharField(blank=True, default="", max_length=15),
        ),
        migrations.AlterField(
            model_name="supplier",
            name="telephone_alternative",
            field=models.CharField(blank=True, default="", max_length=15),
        ),
        migrations.AlterField(
            model_name="units",
            name="extra_value",
            field=models.CharField(default="", max_length=250),
        ),
        migrations.AlterField(
            model_name="units",
            name="field_name",
            field=models.CharField(
                blank=True,
                default="",
                editable=False,
                help_text="Not required",
                max_length=25,
            ),
        ),
        migrations.AlterField(
            model_name="units",
            name="plural_name",
            field=models.CharField(
                default="", max_length=250, verbose_name="Plural name"
            ),
        ),
        migrations.AlterField(
            model_name="unittype",
            name="extra_value",
            field=models.CharField(default="", max_length=250),
        ),
        migrations.AlterField(
            model_name="unittype",
            name="field_name",
            field=models.CharField(
                blank=True,
                default="",
                editable=False,
                help_text="Not required",
                max_length=25,
            ),
        ),
        migrations.AlterField(
            model_name="unittype",
            name="plural_name",
            field=models.CharField(
                default="", max_length=250, verbose_name="Plural name"
            ),
        ),
    ]



from argparse import ArgumentParser

from django.apps import AppConfig as DjangoAppConfig
from django.conf import ENVIRONMENT_VARIABLE, settings
from django.core.exceptions import ImproperlyConfigured
from django.core.management.color import color_style

style = color_style()


class AppConfig(DjangoAppConfig):
    name = "edc_sites"
    verbose_name = "Edc Sites"
    has_exportable_data = True
    default_auto_field = "django.db.models.BigAutoField"
    include_in_administration_section = True

    def ready(self) -> None:
        parser = ArgumentParser()
        _, args = parser.parse_known_args()
        django_settings_module = getattr(settings, ENVIRONMENT_VARIABLE, None)
        if (
            "migrate" in args
            and not django_settings_module
            and not [a for a in args if a.startswith("--settings")]
        ):
            raise ImproperlyConfigured(
                style.ERROR(
                    f"App `{self.verbose_name}` needs access to the correct settings module. "
                    f"Either set `{ENVIRONMENT_VARIABLE}` or pass `--settings` argument when "
                    "running `migrate` from the command line. Expected something like "
                    "`manage.py migrate --settings=my_edc.settings.live."
                )
            )
        if (
            "multisite" in settings.INSTALLED_APPS
            or "multisite.apps.Appconfig" in settings.INSTALLED_APPS
        ) and getattr(settings, "MULTISITE_REGISTER_POST_MIGRATE_SYNC_ALIAS", True):
            raise ImproperlyConfigured(
                style.ERROR(
                    "Multisite post_migrate signal `post_migrate_sync_alias` conflicts "
                    "with a post-migrate signal in 'edc-sites'. To fix the issue, "
                    "`post_migrate_sync_alias` is now registered in "
                    "`edc_appconfig.apps`. To prevent a duplicate registration, you need to "
                    "add `MULTISITE_REGISTER_POST_MIGRATE_SYNC_ALIAS = False` to your "
                    "settings.py. See also edc_appconfig.apps.py"
                )
            )



from django.utils.translation import gettext as _

from .past_date_list_filter import PastDateListFilter


class ReportDateListFilter(PastDateListFilter):
    title = _("Report date")

    parameter_name = "report_datetime"
    field_name = "report_datetime"



from __future__ import annotations

from typing import TYPE_CHECKING

import pandas as pd

from ..get_stock_for_location_df import get_stock_for_location_df

if TYPE_CHECKING:

    from ...models import StockRequest


def get_instock_and_nostock_data(
    stock_request: StockRequest, df_next_scheduled_visits: pd.DataFrame
) -> tuple[pd.DataFrame, pd.DataFrame]:

    df_stock = get_stock_for_location_df(stock_request.location)
    if not df_next_scheduled_visits.empty and not df_stock.empty:
        df_stock = df_next_scheduled_visits.merge(
            df_stock, on="subject_identifier", how="left"
        )
        df_stock["dispensed"] = df_stock["dispensed"].astype("boolean").fillna(False)
        df_stock["stock_qty"] = df_stock["stock_qty"].fillna(0.0)
        for subject_identifier in df_stock.subject_identifier.unique():
            qty_needed = (
                stock_request.containers_per_subject
                - df_stock.loc[
                    (df_stock.subject_identifier == subject_identifier) & ~df_stock.dispensed
                ].shape[0]
            )
            if qty_needed > 0:
                df_stock_needed = df_next_scheduled_visits[
                    df_next_scheduled_visits.subject_identifier == subject_identifier
                ].copy()
                df_stock_needed["code"] = pd.NA
                df_stock_needed = df_stock_needed.loc[df_stock_needed.index.repeat(qty_needed)]
                df_stock = pd.concat([df_stock, df_stock_needed]).reset_index(drop=True)
                df_stock["dispensed"] = df_stock["dispensed"].astype("boolean").fillna(False)
                df_stock["stock_qty"] = 0.0
    else:
        df_stock = df_next_scheduled_visits.copy()
        df_stock["code"] = pd.NA
        df_stock["dispensed"] = False
        df_stock["dispensed"].astype("boolean")
        df_stock["stock_qty"] = 0.0
        df_stock["subject_identifier"] = pd.NA
    df_stock = df_stock.reset_index(drop=True)

    df_instock = (
        df_stock.loc[~(df_stock.code.isna()) & ~df_stock.dispensed]
        .copy()
        .sort_values(by=["subject_identifier"])
        .reset_index(drop=True)
    )
    df_nostock = (
        df_stock.loc[(df_stock.code.isna())]
        .copy()
        .sort_values(by=["subject_identifier"])
        .reset_index(drop=True)
    )
    df_nostock["code"] = df_nostock["code"].fillna(pd.NA)
    return df_instock, df_nostock


__all__ = ["get_instock_and_nostock_data"]



from django.contrib import admin

from edc_model_admin.dashboard import ModelAdminSubjectDashboardMixin
from edc_model_admin.history import SimpleHistoryAdmin

from .admin_site import edc_ltfu_admin
from .modeladmin_mixin import LtfuModelAdminMixin
from .models import Ltfu


@admin.register(Ltfu, site=edc_ltfu_admin)
class LtfuAdmin(LtfuModelAdminMixin, ModelAdminSubjectDashboardMixin, SimpleHistoryAdmin):
    pass



from clinicedc_constants import NO, YES
from django import forms

from edc_form_validators.form_validator import FormValidator


class SubjectLocatorFormValidator(FormValidator):
    def clean(self):
        self.validate_may_call_fields()

        self.required_if(YES, field="may_call_work", field_required="subject_work_place")
        self.not_required_if(
            NO,
            field="may_call_work",
            field_required="subject_work_phone",
            inverse=False,
        )
        self.not_required_if(
            NO, field="may_call_work", field_required="subject_work_cell", inverse=False
        )

        self.required_if(YES, field="home_visit_permission", field_required="physical_address")
        self.required_if(YES, field="may_contact_someone", field_required="contact_name")
        self.required_if(YES, field="contact_name", field_required="contact_rel")
        self.required_if(YES, field="contact_name", field_required="contact_physical_address")

        self.required_if(
            YES, field="may_contact_indirectly", field_required="indirect_contact_name"
        )
        self.required_if(
            YES,
            field="may_contact_indirectly",
            field_required="indirect_contact_relation",
        )
        self.required_if(
            YES,
            field="may_contact_indirectly",
            field_required="indirect_contact_physical_address",
        )

        for field in [
            "indirect_contact_cell",
            "indirect_contact_cell_alt",
            "indirect_contact_phone",
        ]:
            self.not_required_if(
                NO, field="may_contact_indirectly", field_required=field, inverse=False
            )

    def validate_may_call_fields(self):
        validations = {}
        number_fields = ("subject_cell", "subject_phone")
        if self.cleaned_data.get("may_call") == YES:
            if all([self.cleaned_data.get(f) is None for f in number_fields]):
                validations = {k: "This field is required" for k in number_fields}
        elif self.cleaned_data.get("may_call") == NO:
            number_fields = {*number_fields, "subject_cell_alt", "subject_phone_alt"}
            for field in number_fields:
                if self.cleaned_data.get(field):
                    validations.update({field: "This field is not required."})
        if validations:
            raise forms.ValidationError(validations)



export_codenames = [
    "edc_export.display_export_admin_action",
    "edc_export.add_datarequest",
    "edc_export.add_datarequesthistory",
    "edc_export.change_datarequest",
    "edc_export.change_datarequesthistory",
    "edc_export.delete_datarequest",
    "edc_export.delete_datarequesthistory",
    "edc_export.view_datarequest",
    "edc_export.view_datarequesthistory",
    "edc_export.view_historicaldatarequest",
    "edc_export.export_subjectschedulehistory",
    "edc_export.export_visitschedule",
    "edc_export.view_export_dashboard",
]



from edc_model.models import BaseUuidModel
from edc_sites.model_mixins import SiteModelMixin

from ..model_mixins import OnScheduleModelMixin


class OnSchedule(SiteModelMixin, OnScheduleModelMixin, BaseUuidModel):
    """A model used by the system. Auto-completed by subject_consent."""

    class Meta(OnScheduleModelMixin.Meta, BaseUuidModel.Meta):
        pass



ATTENDED_APPT = "attended"
CANCELLED_APPT = "cancelled"
COMPLETE_APPT = "done"
INCOMPLETE_APPT = "incomplete"
IN_PROGRESS_APPT = "in_progress"
LAST_MONTH = "last_month"
EXTENDED_APPT = "extended"
MISSED_APPT = "missed"
NEW_APPT = "new"
NEXT_MONTH = "next_month"
ONTIME_APPT = "ontime"
SCHEDULED_APPT = "scheduled"
SKIPPED_APPT = "skipped"
UNSCHEDULED_APPT = "unscheduled"

INVALID_APPT_DATE = "invalid_appt_datetime"
INVALID_APPT_STATUS = "invalid_appt_status"
INVALID_APPT_REASON = "invalid_appt_reason"
INVALID_PREVIOUS_APPOINTMENT_NOT_UPDATED = "previous_appointment_not_updated"
INVALID_PREVIOUS_VISIT_MISSING = "previous_visit_missing"
INVALID_MISSED_APPT_NOT_ALLOWED = "invalid_missed_appt_not_allowed"
INVALID_APPT_STATUS_AT_BASELINE = "invalid_appt_status_at_baseline"
INVALID_SKIPPED_APPT_NOT_ALLOWED_AT_BASELINE = "invalid_skipped_appt_not_allowed_at_baseline"
INVALID_APPT_TIMING = "invalid_appt_timing"
INVALID_APPT_TIMING_CRFS_EXIST = "invalid_appt_timing_crfs_exist"
INVALID_APPT_TIMING_REQUISITIONS_EXIST = "invalid_appt_timing_requisitions_exist"

LT_30_DAYS = "LT_30_DAYS"
GTE_30_TO_60_DAYS = "GTE_30_TO_60_DAYS"
GTE_60_TO_90_DAYS = "GTE_60_TO_90_DAYS"
GTE_90_TO_180_DAYS = "GTE_90_TO_180_DAYS"
GTE_180 = "GTE_180"



from clinicedc_constants import TBD
from django.core.validators import (
    MaxLengthValidator,
    MinLengthValidator,
    RegexValidator,
)
from django.db import models
from django.utils import timezone

from edc_action_item.models.action_model_mixin import ActionModelMixin
from edc_constants.choices import YES_NO_TBD
from edc_identifier.managers import SubjectIdentifierManager
from edc_identifier.model_mixins import NonUniqueSubjectIdentifierFieldMixin
from edc_model.models.base_uuid_model import BaseUuidModel
from edc_sites.managers import CurrentSiteManager
from edc_sites.model_mixins import SiteModelMixin

from ..constants import UNBLINDING_REQUEST_ACTION
from .unblinding_user import UnblindingRequestorUser


class UnblindingRequest(
    NonUniqueSubjectIdentifierFieldMixin,
    SiteModelMixin,
    ActionModelMixin,
    BaseUuidModel,
):
    action_name = UNBLINDING_REQUEST_ACTION

    report_datetime = models.DateTimeField(
        verbose_name="Report Date and Time", default=timezone.now
    )

    initials = models.CharField(
        verbose_name="Subject's initials",
        max_length=3,
        validators=[
            RegexValidator("[A-Z]{1,3}", "Invalid format"),
            MinLengthValidator(2),
            MaxLengthValidator(3),
        ],
        help_text="Use UPPERCASE letters only. May be 2 or 3 letters.",
    )

    requestor = models.ForeignKey(
        UnblindingRequestorUser,
        related_name="+",
        on_delete=models.PROTECT,
        verbose_name="Unblinding requested by",
        help_text="Choose a name from the list",
    )

    unblinding_reason = models.TextField(verbose_name="Reason for unblinding")

    approved = models.CharField(max_length=15, default=TBD, choices=YES_NO_TBD)

    approved_datetime = models.DateTimeField(null=True)

    objects = SubjectIdentifierManager()

    on_site = CurrentSiteManager()

    def natural_key(self):
        return (self.action_identifier,)

    class Meta(BaseUuidModel.Meta, NonUniqueSubjectIdentifierFieldMixin.Meta):
        verbose_name = "Unblinding Request"
        verbose_name_plural = "Unblinding Requests"
        indexes = (
            models.Index(fields=["subject_identifier", "action_identifier", "site", "id"]),
        )



from django.db import models
from django.db.models.deletion import PROTECT
from django.utils import timezone

from edc_model.models import BaseUuidModel, HistoricalRecords
from edc_model.validators import datetime_not_future
from edc_sites.managers import CurrentSiteManager
from edc_sites.model_mixins import SiteModelMixin

from .aliquot import Aliquot


class OrderManager(models.Manager):
    use_in_migrations = True

    def get_by_natural_key(self, report_datetime, aliquot_identifier):
        return self.get(
            report_datetime=report_datetime,
            aliquot__aliquot_identifier=aliquot_identifier,
        )


class Order(SiteModelMixin, BaseUuidModel):
    aliquot = models.ForeignKey(Aliquot, on_delete=PROTECT)

    order_identifier = models.CharField(max_length=25, editable=False, unique=True)

    order_datetime = models.DateTimeField(
        default=timezone.now, validators=[datetime_not_future]
    )

    panel_name = models.CharField(max_length=25)

    objects = OrderManager()

    on_site = CurrentSiteManager()

    history = HistoricalRecords()

    def natural_key(self):
        return self.report_datetime, *self.aliquot.natural_key()

    natural_key.dependencies = ("edc_lab.aliquot", "sites.Site")

    class Meta(BaseUuidModel.Meta):
        verbose_name = "Order"



from typing import Any

from django import forms


def get_field_or_raise(name: str, msg: str, cleaned_data: dict, instance) -> Any:
    """Returns a field value from cleaned_data if the key
    exists, or from the model instance.

    Raises if field value is none.
    """
    if name in cleaned_data and not cleaned_data.get(name):
        raise forms.ValidationError({"__all__": msg})
    value = cleaned_data.get(name, getattr(instance, name))
    if not value:
        raise forms.ValidationError({"__all__": msg})
    return value



import sys

from django.conf import settings

from ..models import QueryRule
from .rule_runner import RuleRunner

DATA_MANAGER_ENABLED = getattr(settings, "DATA_MANAGER_ENABLED", True)


def update_query_rules(pks=None, verbose=None):
    total_created = 0
    total_resolved = 0
    if verbose:
        sys.stdout.write("\nRunning query rules:\n")
    if not DATA_MANAGER_ENABLED:
        sys.stdout.write(
            "Disabled. To enable, set settings.DATA_MANAGER_ENABLED = True. Done\n"
        )
    else:
        opts = dict(active=True)
        if pks:
            opts.update(id__in=pks)
        query_rules = QueryRule.objects.filter(**opts)
        sys.stdout.write(
            f"  Found {query_rules.count()} rule{'s'[:query_rules.count() ^ 1]} to run.\n"
        )
        for index, query_rule_obj in enumerate(query_rules.all()):
            if verbose:
                sys.stdout.write(f"  {index + 1}. {query_rule_obj}\n")
            rule_runner = RuleRunner(query_rule_obj, verbose=verbose)
            created, resolved = rule_runner.run()
            total_created += created
            total_resolved += resolved
        if verbose:
            sys.stdout.write(f"  total queries created {total_created}\n")
            sys.stdout.write(f"  total queries resolved {total_resolved}\n")
    sys.stdout.write("Done.\n")
    return {"created": total_created, "resolved": total_resolved}



import pandas as pd
from django.apps import apps as django_apps
from django.contrib.sites.models import Site
from django_pandas.io import read_frame

from ..utils import convert_dates_from_model


def get_eos(
    model: str,
    subject_identifiers: list[str] | None = None,
    normalize: bool | None = None,
    localize: bool | None = None,
    fields: list[str] | None = None,
    all_fields: bool | None = None,
) -> pd.DataFrame:
    normalize = True if normalize is None else normalize
    localize = True if localize is None else localize
    model_cls = django_apps.get_model(model)

    if all_fields:
        fields = [fld.name for fld in model_cls._meta.get_fields()]
    elif not fields:
        fields = ["subject_identifier", "offstudy_datetime", "offstudy_reason", "site"]

    if subject_identifiers:
        qs = model_cls.objects.values(*fields).filter(
            subject_identifier__in=subject_identifiers
        )
    else:
        qs = model_cls.objects.values(*fields).all()

    df = read_frame(qs, verbose=True)

    df = convert_dates_from_model(df, model_cls, normalize=normalize, localize=localize)

    df["site_id"] = df["site"].map({obj.domain: obj.id for obj in Site.objects.all()})
    df = df.drop(columns=["site"])

    df.sort_values(by=["subject_identifier"])
    df = df.reset_index(drop=True)
    return df



from __future__ import annotations

from typing import TYPE_CHECKING

from django.contrib.sites.models import Site
from reportlab.graphics.barcode.widgets import BarcodeStandard39
from reportlab.graphics.charts.textlabels import Label as RlLabel
from reportlab.graphics.shapes import Drawing, String

if TYPE_CHECKING:
    # you may also refer to models in clinicedc/edc-pharmacy
    class Container:
        qty: int

    class Request:
        container: Container

    class RequestItem:
        subject_identifier: str
        code: str
        gender: str
        sid: int | str
        site: Site
        request: Request


def draw_callable_example(
    label: Drawing,
    width: int | float,
    height: int | float,
    obj: RequestItem,
) -> Drawing:
    """Callable to draw a single study medication label given a model
    instance `obj`

    Prints a code39 barcode for the code attribute of obj.
    """
    br = BarcodeStandard39(
        humanReadable=True, checksum=False, barHeight=30, barWidth=0.7, gap=1.7
    )
    br.value = obj.code
    br.x = width - 140
    br.y = 25
    label.add(br)
    label.add(String(15, height - 20, f"TEST!! Study - {obj.site.name.title()}", fontSize=10))
    label.add(
        String(
            width - 110,
            height - 40,
            f"{obj.subject_identifier}{obj.gender}",
            fontSize=12,
        )
    )
    label.add(String(15, height - 40, "Medication for the research", fontSize=10))
    label.add(String(15, height - 50, "trial TEST!!.", fontSize=10))
    label.add(String(15, height - 70, "Take 4 pills only at night.", fontSize=10))
    label.add(String(15, 20, f"{obj.request.container.qty} tabs", fontSize=10))
    lab = RlLabel(x=width - 20, y=40, fontSize=10, angle=90)
    lab.setText(str(obj.sid))
    label.add(lab)
    return label



from __future__ import annotations

from typing import TYPE_CHECKING, Any

from django.db.models.aggregates import Count

from edc_appointment.view_mixins import AppointmentViewMixin
from edc_dashboard.url_names import url_names
from edc_dashboard.view_mixins import EdcViewMixin
from edc_listboard.view_mixins import ListboardFilterViewMixin, SearchFormViewMixin
from edc_listboard.views import ListboardView
from edc_metadata.view_mixins import MetadataViewMixin
from edc_navbar.view_mixin import NavbarViewMixin
from edc_subject_dashboard.view_mixins import (
    RegisteredSubjectViewMixin,
    SubjectVisitViewMixin,
)
from edc_visit_schedule.view_mixins import VisitScheduleViewMixin
from edc_visit_tracking.utils import get_related_visit_model

if TYPE_CHECKING:
    from edc_model.models import BaseUuidModel
    from edc_sites.model_mixins import SiteModelMixin
    from edc_visit_tracking.model_mixins import VisitModelMixin as Base

    class RelatedVisitModel(SiteModelMixin, Base, BaseUuidModel):
        pass


class SubjectReviewListboardView(
    EdcViewMixin,
    NavbarViewMixin,
    MetadataViewMixin,
    AppointmentViewMixin,
    SubjectVisitViewMixin,
    VisitScheduleViewMixin,
    RegisteredSubjectViewMixin,
    ListboardFilterViewMixin,
    SearchFormViewMixin,
    ListboardView,
):
    listboard_model = get_related_visit_model()
    navbar_selected = "subject_review"

    listboard_template = "subject_review_listboard_template"
    listboard_url = "subject_review_listboard_url"
    listboard_back_url = "subject_review_listboard_url"
    listboard_panel_style = "default"
    listboard_panel_title = "Subject Review"
    listboard_view_permission_codename = "edc_review_dashboard.view_subject_review_listboard"

    navbar_selected_item = "subject_review"
    ordering = ("subject_identifier", "visit_code", "visit_code_sequence")
    paginate_by = 25
    paginator_url = "subject_review_listboard_url"
    search_form_url = "subject_review_listboard_url"
    search_fields = (
        "subject_identifier",
        "visit_code",
        "user_created",
        "user_modified",
    )

    # Attribute to call SubjectReviewListboardView.urls in urls.py
    # See UrlRequestContextMixin and UrlConfig
    urlconfig_getattr = "review_listboard_urls"

    def get_context_data(self, **kwargs) -> dict[str, Any]:
        kwargs.update(
            subject_dashboard_url=url_names.get("subject_dashboard_url"),
        )
        return super().get_context_data(**kwargs)

    def get_queryset(self):
        queryset = super().get_queryset()
        if not self.subject_identifier:
            queryset = (
                queryset.order_by("subject_identifier")
                .values("subject_identifier")
                .annotate(visit_count=Count("subject_identifier"))
            )
        return queryset



# Generated by Django 4.2.7 on 2023-12-04 02:32

from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        (
            "edc_listboard",
            "0004_alter_listboard_options_listboard_locale_created_and_more",
        ),
    ]

    operations = [
        migrations.RemoveIndex(
            model_name="listboard",
            name="edc_listboa_modifie_a5fdf8_idx",
        ),
        migrations.AddIndex(
            model_name="listboard",
            index=models.Index(
                fields=["modified", "created"], name="edc_listboa_modifie_e63312_idx"
            ),
        ),
    ]



from __future__ import annotations

import contextlib
from typing import TYPE_CHECKING, Any

from django.apps import apps as django_apps
from django.contrib import messages
from django.core.exceptions import ObjectDoesNotExist
from django.utils.translation import gettext as _

from edc_sites.site import sites

from ..constants import (
    CANCELLED_APPT,
    COMPLETE_APPT,
    IN_PROGRESS_APPT,
    INCOMPLETE_APPT,
    NEW_APPT,
    SKIPPED_APPT,
)

if TYPE_CHECKING:
    from django.db.models import QuerySet

    from ..models import Appointment


class AppointmentViewMixin:
    """A view mixin to handle appointments on the dashboard."""

    def __init__(self, **kwargs):
        self._appointment = None
        self._appointments = None
        self.appointment_model: str = "edc_appointment.appointment"
        self.appointment_id: str | None = None
        super().__init__(**kwargs)

    def get(self, request, *args, **kwargs):
        self.appointment_id = kwargs.get("appointment")
        return super().get(request, *args, **kwargs)

    def get_context_data(self, **kwargs) -> dict[str, Any]:
        if self.appointment:
            if self.appointment.appt_status != IN_PROGRESS_APPT:
                message = _(
                    'You have selected an appointment that is no longer "in progress". '
                    "Refer to the schedule for the appointment that is "
                    'currently "in progress".'
                )
                self.message_user(message, level=messages.WARNING)
            if self.appointment.related_visit:
                report_datetime = self.appointment.related_visit.report_datetime
                kwargs.update(report_datetime=report_datetime)
        has_call_manager = bool(django_apps.app_configs.get("edc_call_manager"))
        kwargs.update(
            appointment=self.appointment,
            appointments=self.appointments,
            CANCELLED_APPT=CANCELLED_APPT,
            COMPLETE_APPT=COMPLETE_APPT,
            INCOMPLETE_APPT=INCOMPLETE_APPT,
            IN_PROGRESS_APPT=IN_PROGRESS_APPT,
            NEW_APPT=NEW_APPT,
            SKIPPED_APPT=SKIPPED_APPT,
            has_call_manager=has_call_manager,
        )
        return super().get_context_data(**kwargs)

    @property
    def appointment_model_cls(self) -> Appointment:
        return django_apps.get_model(self.appointment_model)

    @property
    def appointment_options(self) -> dict[str, Any]:
        opts = {}
        if self.kwargs.get("appointment"):
            opts = dict(id=self.kwargs.get("appointment"))
        elif (
            self.kwargs.get("subject_identifier")
            and self.kwargs.get("visit_schedule_name")
            and self.kwargs.get("schedule_name")
            and self.kwargs.get("visit_code")
        ):
            visit_code_sequence = self.kwargs.get("visit_code_sequence") or 0
            opts = dict(
                subject_identifier=self.kwargs.get("subject_identifier"),
                visit_schedule_name=self.kwargs.get("visit_schedule_name"),
                schedule_name=self.kwargs.get("schedule_name"),
                visit_code=self.kwargs.get("visit_code"),
                visit_code_sequence=visit_code_sequence,
                site_id__in=sites.get_site_ids_for_user(request=self.request),
            )
        return opts

    @property
    def appointment(self) -> Appointment:
        if not self._appointment and self.appointment_id:
            try:
                self._appointment = self.appointment_model_cls.objects.get(
                    id=self.appointment_id
                )
            except ObjectDoesNotExist:
                if opts := self.appointment_options:
                    with contextlib.suppress(ObjectDoesNotExist):
                        self._appointment = self.appointment_model_cls.objects.get(**opts)
        return self._appointment

    @property
    def appointments(self) -> QuerySet[Appointment]:
        """Returns a Queryset of all appointments for this subject."""
        if not self._appointments:
            self._appointments = self.appointment_model_cls.objects.filter(
                subject_identifier=self.subject_identifier,
                site_id__in=sites.get_site_ids_for_user(request=self.request),
            ).order_by("timepoint", "visit_code_sequence")

        return self._appointments



class Fieldlist:
    def __init__(
        self,
        insert_fields=None,
        remove_fields=None,
        insert_after=None,
        section=None,
        **kwargs,
    ):
        self.insert_fields = insert_fields
        self.remove_fields = remove_fields
        self.insert_after = insert_after
        self.section = section


class Remove:
    def __init__(self, *fields, section=None, **kwargs):
        self.remove_fields = fields
        self.section = section


class Insert:
    def __init__(self, *fields, after=None, section=None, **kwargs):
        self.insert_fields = fields
        self.insert_after = after
        self.section = section



from __future__ import annotations

from django.apps import apps as django_apps
from django.db import models


class CrfLookupError(Exception):
    pass


class CrfModelNotProxyModelError(Exception):
    pass


class Crf:
    def __init__(
        self,
        *,
        show_order: int,
        model: str,
        required: bool | None = None,
        additional: bool | None = None,
        site_ids: list[int] | None = None,
        shares_proxy_root: bool | None = None,
    ) -> None:
        self.additional = additional
        self.model = model.lower()
        self.required = True if required is None else required
        self.show_order = show_order
        self.site_ids = site_ids or []
        self.shares_proxy_root = shares_proxy_root or False

        if self.shares_proxy_root and not self.model_cls._meta.proxy:
            raise CrfModelNotProxyModelError(
                "Invalid use of `shares_proxy_root=True`. "
                f"CRF model is not a proxy model. Got {self.full_name}."
            )

    def __repr__(self) -> str:
        return f"{self.__class__.__name__}({self.show_order}, {self.model}, {self.required})"

    def __str__(self) -> str:
        required = "Required" if self.required else ""
        return f"{self.model} {required}"

    @property
    def name(self) -> str:
        return f"{self.model}.{'required' if self.required else 'not_required'}"

    def validate(self) -> None:
        """Raises an exception if the model class lookup fails."""
        try:
            self.get_model_cls()
        except LookupError as e:
            raise CrfLookupError(e) from e

    def get_model_cls(self) -> type[models.Model]:
        return self.model_cls

    @property
    def model_cls(self) -> type[models.Model]:
        return django_apps.get_model(self.model)

    @property
    def verbose_name(self) -> str:
        return self.model_cls._meta.verbose_name

    @property
    def full_name(self):
        return self.model



from django.contrib import messages
from django.db.models.deletion import ProtectedError

from edc_dashboard.view_mixins import EdcViewMixin
from edc_lab import SHIPPED
from edc_lab.labels import ManifestLabel
from edc_lab.models import Aliquot, Box, Manifest
from edc_label import LabPrintersMixin, add_job_results_to_messages

from ...view_mixins import ManifestViewMixin
from .action_view import ActionView


class ManifestView(EdcViewMixin, ManifestViewMixin, LabPrintersMixin, ActionView):
    post_action_url = "manifest_listboard_url"
    valid_form_actions = (
        "remove_selected_items",
        "print_labels",
        "ship_selected_items",
    )
    label_cls = ManifestLabel

    def process_form_action(self, request=None):
        if not self.selected_items:
            message = "Nothing to do. No items have been selected."
            messages.warning(request, message)
        elif self.action == "remove_selected_items":
            self.remove_selected_items()
        elif self.action == "print_labels":
            job_result = self.print_labels(pks=self.selected_items, request=request)
            if job_result:
                add_job_results_to_messages(request, [job_result])
        elif self.action == "ship_selected_items":
            self.ship_selected_items()

    def remove_selected_items(self):
        """Deletes the selected items, if allowed."""
        try:
            deleted = Manifest.objects.filter(
                pk__in=self.selected_items, shipped=False
            ).delete()
            message = f"{deleted[0]} manifest(s) have been removed."
            messages.success(self.request, message)
        except ProtectedError:
            message = "Unable to remove. Manifest is not empty."
            messages.error(self.request, message)

    def ship_selected_items(self):
        """Flags selected items as shipped."""
        for manifest in Manifest.objects.filter(pk__in=self.selected_items):
            if manifest.shipped:
                message = (
                    f"Manifest has already been shipped. Got {manifest.manifest_identifier}."
                )
                messages.error(self.request, message)
            else:
                boxes = Box.objects.filter(
                    box_identifier__in=[
                        obj.identifier for obj in manifest.manifestitem_set.all()
                    ]
                )
                boxes.update(status=SHIPPED)
                for box in boxes:
                    aliquots = Aliquot.objects.filter(
                        aliquot_identifier__in=[
                            obj.identifier for obj in box.boxitem_set.all()
                        ]
                    )
                    aliquots.update(shipped=True)
                manifest.shipped = True
                manifest.save()

                message = f"Manifest {manifest.manifest_identifier} has been shipped."
                messages.success(self.request, message)



from __future__ import annotations

import sys
from typing import TYPE_CHECKING

import requests
from django.conf import settings
from django.core.exceptions import ValidationError

from .utils import get_email_enabled

if TYPE_CHECKING:
    from django.contrib.auth.models import User
    from requests.models import Response


class EmailNotEnabledError(ValidationError):
    pass


class UserEmailError(ValidationError):
    pass


class MailingListManager:
    """A class to create (and update) mailing lists, subscribe,
    unsubscribe members, etc via the MAILGUN API.

    If this is a test / UAT, the mailing list names from settings
    are automatically prefixed with 'test'.
    """

    url = "https://api.mailgun.net/v3/lists"
    api_url_attr = "MAILGUN_API_URL"
    api_key_attr = "MAILGUN_API_KEY"

    def __init__(self, address: str = None, name: str = None, display_name: str = None):
        self._api_key: str | None = None
        self._api_url: str | None = None
        self.address = address  # mailing list address
        self.display_name = display_name
        self.email_enabled: bool = get_email_enabled()
        self.name = name

    @property
    def api_url(self) -> str | None:
        """Returns the api_url or None."""
        if not self._api_url:
            error_msg = (
                f"Email is enabled but API_URL is not set. See settings.{self.api_url_attr}"
            )
            try:
                self._api_url = getattr(settings, self.api_url_attr)
            except AttributeError:
                raise EmailNotEnabledError(error_msg, code="api_url_attribute_error")
            else:
                if not self._api_url:
                    raise EmailNotEnabledError(error_msg, code="api_url_is_none")
        return self._api_url

    @property
    def api_key(self) -> str | None:
        """Returns the api_key or None."""
        if not self._api_key:
            error_msg = (
                f"Email is enabled but API_KEY is not set. See settings.{self.api_key_attr}"
            )
            try:
                self._api_key = getattr(settings, self.api_key_attr)
            except AttributeError:
                raise EmailNotEnabledError(error_msg, code="api_key_attribute_error")
            else:
                if not self._api_key:
                    raise EmailNotEnabledError(error_msg, code="api_key_is_none")
        return self._api_key

    def subscribe(self, user: User, verbose: bool | None = None) -> Response:
        """Returns a response after attempting to subscribe
        a member to the list.
        """
        if not self.email_enabled:
            raise EmailNotEnabledError("Email is not enabled. See settings.EMAIL_ENABLED")
        if not user.email:
            raise UserEmailError(f"User {user}'s email address is not defined.")
        response = requests.post(
            f"{self.api_url}/{self.address}/members",
            auth=("api", self.api_key),
            data={
                "subscribed": True,
                "address": user.email,
                "name": f"{user.first_name} {user.last_name}",
                "description": f"{user.userprofile.job_title or ''}",
                "upsert": "yes",
            },
            timeout=10,
        )
        if verbose:
            self._output_response_message("subscribe", response)
        return response

    def unsubscribe(self, user, verbose: bool | None = None) -> Response:
        """Returns a response after attempting to unsubscribe
        a member from the list.
        """
        if not self.email_enabled:
            raise EmailNotEnabledError("Email is not enabled. See settings.EMAIL_ENABLED")
        response = requests.put(
            f"{self.api_url}/{self.address}/members/{user.email}",
            auth=("api", self.api_key),
            data={"subscribed": False},
            timeout=10,
        )
        if verbose:
            self._output_response_message("unsubscribe", response)
        return response

    def _output_response_message(self, action: str, response: Response) -> None:
        try:
            email = response.json()["member"]["address"]
            message = response.json()["message"]
            subscribed = response.json()["member"]["subscribed"]
        except KeyError:
            sys.stdout.write(
                f"{action.title()} failed. Got response={response.status_code} "
                f"{response.json()!s}"
            )
        else:
            sys.stdout.write(
                f"{action.title()} for {email} from {self.address} successful. "
                f"Got response={response.status_code}. {message} "
                f"subscribed={subscribed}.\n"
            )

    def create(self, verbose: bool | None = None) -> Response:
        """Returns a response after attempting to create the list."""
        if not self.email_enabled:
            raise EmailNotEnabledError("Email is not enabled. See settings.EMAIL_ENABLED")
        response = requests.post(
            self.api_url,
            auth=("api", self.api_key),
            data={
                "address": self.address,
                "name": self.name,
                "description": self.display_name,
            },
            timeout=10,
        )
        if verbose:
            sys.stdout.write(
                f"Creating mailing list {self.address}. Got response={response.status_code}.\n"
            )
        return response

    def delete(self) -> Response:
        """Returns a response after attempting to delete the list."""
        if not self.email_enabled:
            raise EmailNotEnabledError("Email is not enabled. See settings.EMAIL_ENABLED")
        return requests.delete(
            f"{self.api_url}/{self.address}", auth=("api", self.api_key), timeout=10
        )

    def delete_member(self, user: User) -> Response:
        """Returns a response after attempting to remove
        a member from the list.
        """
        if not self.email_enabled:
            raise EmailNotEnabledError("Email is not enabled. See settings.EMAIL_ENABLED")
        return requests.delete(
            f"{self.api_url}/{self.address}/members/{user.email}",
            auth=("api", self.api_key),
            timeout=10,
        )



from edc_auth.constants import AUDITOR_ROLE, CLINICIAN_ROLE
from edc_auth.site_auths import site_auths
from edc_data_manager.auth_objects import DATA_MANAGER_ROLE

from .auth_objects import (
    EDC_FORM_RUNNERS,
    EDC_FORM_RUNNERS_SUPER,
    EDC_FORM_RUNNERS_VIEW,
    codenames,
)


def update_site_auths():
    site_auths.add_group(*codenames, name=EDC_FORM_RUNNERS_VIEW, view_only=True)
    site_auths.add_group(*codenames, name=EDC_FORM_RUNNERS, no_delete=True)
    site_auths.add_group(*codenames, name=EDC_FORM_RUNNERS_SUPER)

    site_auths.update_role(EDC_FORM_RUNNERS_VIEW, name=AUDITOR_ROLE)
    site_auths.update_role(EDC_FORM_RUNNERS, name=CLINICIAN_ROLE)
    site_auths.update_role(EDC_FORM_RUNNERS_SUPER, name=DATA_MANAGER_ROLE)


update_site_auths()



# -*- coding: utf-8 -*-
# Generated by Django 1.9.7 on 2016-06-25 09:38
from __future__ import unicode_literals

import uuid

from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [("edc_identifier", "0002_auto_20160505_1435")]

    operations = [
        migrations.AlterField(
            model_name="identifiertracker",
            name="id",
            field=models.UUIDField(
                blank=True,
                default=uuid.uuid4,
                editable=False,
                help_text="System auto field. UUID primary key.",
                primary_key=True,
                serialize=False,
            ),
        ),
        migrations.AlterField(
            model_name="subjectidentifier",
            name="id",
            field=models.UUIDField(
                blank=True,
                default=uuid.uuid4,
                editable=False,
                help_text="System auto field. UUID primary key.",
                primary_key=True,
                serialize=False,
            ),
        ),
    ]



from .notification import Notification
from .site_notifications import site_notifications


class RegisterNotificationError(Exception):
    pass


def register(**kwargs):
    """Registers a notification_cls."""

    def _wrapper(notification_cls):
        if not issubclass(notification_cls, (Notification,)):
            raise RegisterNotificationError(
                f"Wrapped class must be a 'Notification' class. "
                f"Got '{notification_cls.__name__}'"
            )
        site_notifications.register(notification_cls=notification_cls)
        return notification_cls

    return _wrapper



from django import forms


class ActionItemFormMixin:
    """Declare with forms.ModelForm."""

    class Meta:
        help_text = {
            "subject_identifier": "(read-only)",
            "action_identifier": "(read-only)",
        }
        widgets = {
            "subject_identifier": forms.TextInput(attrs={"readonly": "readonly"}),
            "action_identifier": forms.TextInput(attrs={"readonly": "readonly"}),
        }


class ActionItemCrfFormMixin:
    """Declare with forms.ModelForm."""

    class Meta:
        help_text = {"action_identifier": "(read-only)"}
        widgets = {
            "action_identifier": forms.TextInput(attrs={"readonly": "readonly"}),
        }



from clinicedc_constants import OTHER

list_data = {
    "edc_adherence.nonadherencereasons": [
        ("forget_to_take", "I sometimes forget to take my pills"),
        ("dont_like_taking", "I don't like taking my pills"),
        ("make_me_ill", "My pills sometimes make me feel sick"),
        ("misplaced_pills", "I sometimes misplace my pills"),
        ("dont_believe_pills_help", "I don't believe my pills are helping me"),
        ("dont_believe_pills_needed", "I don't believe I need to take my pills"),
        ("not_feeling_well", "I have not been feeling well"),
        (OTHER, "Other, please specify ..."),
    ],
}



from __future__ import annotations

from dataclasses import dataclass, field


class AlreadyRegistered(Exception):  # noqa: N818
    pass


class InvalidDashboardUrlName(Exception):  # noqa: N818
    pass


@dataclass
class UrlNames:
    registry: dict[str, str] = field(default_factory=dict)

    def register(
        self,
        key: str,
        namespace: str | None = None,
        url: str | None = None,
        url_with_namespace: str | None = None,
    ) -> None:
        url_with_namespace = url_with_namespace or f"{namespace}:{url}"
        if key in self.registry:
            raise AlreadyRegistered(
                "Url already registered with url_names. "
                f"See {key}:{self.registry[key]}. Got {url_with_namespace}."
            )
        self.registry.update({key: url_with_namespace})

    def register_from_dict(self, **urldata: str) -> None:
        for key, url_with_namespace in urldata.items():
            try:
                namespace, url = url_with_namespace.split(":")
            except ValueError:
                namespace, url = url_with_namespace, None
            self.register(key, namespace, url=url)

    def all(self) -> dict[str, str]:
        return self.registry

    def get(self, key: str) -> str:
        if key not in self.registry:
            raise InvalidDashboardUrlName(
                f"Invalid key for url_names. Expected one of {self.registry.keys()}. "
                f"Got '{key}'."
            )
        return self.registry.get(key)

    def get_or_raise(self, key: str) -> str:
        return self.get(key)


url_names = UrlNames()



from django.core.management.base import BaseCommand, CommandError

from edc_list_data.site_list_data import SiteListDataError, site_list_data


class Command(BaseCommand):
    help = "Populates list data and other static model data from list_data.py"
    module_name = "list_data"

    def handle(self, *args, **options):  # noqa: ARG002
        try:
            site_list_data.autodiscover()
        except SiteListDataError as e:
            raise CommandError(e) from e



from .bulk_create_stock_request_items import bulk_create_stock_request_items
from .get_instock_and_nostock_data import get_instock_and_nostock_data

__all__ = ["bulk_create_stock_request_items", "get_instock_and_nostock_data"]
